
* Computes Jackknife Standard Errors for Regression Coefficients using fisrt plausible
  value    ;
* It also computes the total variance for regression coefficients using all Plausible values;
%macro stattool(wgt=popwt, rwgt=, nrep=, npv=1, study=, cntryno=, infile=, c=,
                byvar=, byvarpv=, ivar=, ivarpv=, dvar=, dvarpv=, clvar=, clvarpv=, quant=, quantpv=, prefix=9,
                ref=default, type=, method=, stat=mean, where=, fay=0.7, ddf=, clev=.975, jkz=jkzone, jki=jkrep,
                compare1=no, to1=no, label1=-, compare2=no, to2=no, label2=-, compare3=no, to3=no, label3=-, compare4=no, to4=no, label4=-,
                compare5=no, to5=no, label5=-, compare6=no, to6=no, label6=-, compare7=no, to7=no, label7=-, compare8=no, to8=no, label8=-,
                compare9=no, to9=no, label9=-, criter1=, criter2=, criter3=, pctlbrk=, out=, interaction=, option=, event=, items=, pts_2=,
                international = 0, aright=, bright=, cright=, dright=, eright=, o1right=, o2right=, scored=0, pct_correct=0, pct_cond=);

options nocenter nodate;
* WGT= popwt     : The sampling weight (POPWT by default) ;
* rwgt=          : The root of the replicate weight variable name ;
*                : For example, if the replicate weights are given by rep01-rep30 then rwgt = rep ;
* nrep=          : The number of sets of replicate weights (30 by default);
* NPV= 1         : The number of sets of Plausible Values (1 by default);
* c =            : For stidies other than ALL, PISA, TIMSS and PIRLS, specify the c value in order to compute the variance;
* study= all     : The name of the study (ALL by default) TIMSS, PIRLS, PISA and OTHERS are the other possibilities;
* cntryno= cntrid: The id number identifying the country (CNTRID by default);
* INFILE=        : The temporary SAS file with the data. INFILE = setup is reserved for the average percent correct estmation only;
* byvar =        : The subpopulation variable(s);
*                : Note that byvar= x1-x4 is not allowed, you would have to put byvar= x1 x2 x3 x4 ;
*                : If a byvar variable is using plausible values then specify that variable by its root only;
*                : For example, dealing with xdoc1, xdoc2, ..., xdoc5, byvar becomes byvar= xdoc;
* byvarpv =      : The subpopulation variable(s) plausible value indicator;
*                : For example, if byvar= d1 xdoc and xdoc is based on plausible values but not d1, then byvarpv = 0 1 ;
* ivar =         : The independent variable(s) (full description needed [x1-x9 not acceptable]);
*                : when dealing with plausible value variables, only the root is required. ;
* ivarpv = 0     : The independent variable(s) plausible value indicator;
*                : For example, if ivar= x1 xdoc and xdoc is based on plausible values but not x1, then ivarpv = 0 1 ;
* dvar =         : The dependent variable ;
*                : when dealing with plausible value variables, only the root is required. ;
* dvarpv =       : The dependent variable plausible value indicator;
*                : For example, if dvar= prose and prose is the root for prose1 to prose5, then ivarpv = 1 ;
* clvar =        : The automatic dummy coded variable(s) (full description needed [x1-x9 not acceptable]);
*                : when dealing with plausible value variables, only the root is required. ;
*                : Required only when dealing with standard regression techniques ;
*                : Identifies which of the ivar variables are to be treated as dummy variables in a regression ;
* clvarpv =      : The automated dummy coded variable(s) plausible value indicator;
*                : For example, if clvar= x1 xdoc and xdoc is based on plausible values but not x1, then clvarpv = 0 1 ;
* ref=default    : The reference variable (clvar by default, required only with regression techniques);
*                : For example, if clvar=sex age then if ref=1 3 then sex=1 and age=3 will be the reference
                   in the regresion analysis rather than the highest values of sex and age);
* type=          : The type of logistic regression required (ALOGIT specifies response functions as adjacent-category
                   logits of the marginal probabilities for each of the dependent variables - ;
*                   CLOGIT specifies that the response functions are cumulative logits of the marginal probabilities
                    for each of the dependent variables. See SAS CATMOD RESPONSE statement for more details.);
* Method =       : The required method - REGR for a standard regression, MEAN for means, PERC for percentiles,
                   Crosstabs for Crosstabs (percentages), POPEST for population counts;
*                  Logistic and Multinomial for logistic and multinomial logistic regression;
* Interaction =	: It is possible to specify interaction terms in any of the regression model specified by the METHOD parameter,
                   (regr, logistic, multinomial);
* option =			: It is possible to specify no intercept term in any of the regression model specified by the METHOD parameter,
                   (regr, logistic, multinomial). Just specify OPTION=noint ;
* event =			: It is possible to change the dependent reference category when logistic regression are requested,
                   (logistic, multinomial, survival, probit). ;
* compare1=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To1=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label1=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare2=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To2=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label2=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare3=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To3=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label3=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare4=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To4=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label4=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare5=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To5=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label5=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare6=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To6=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label6=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare7=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To7=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label7=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare8=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To8=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label8=-       : When tests are required, this parameter gives the name you want to give to your test;
* compare9=no    : When tests are required, compare1 identifies the cell we want to compare. For example, specify
*                  "compare = itsex=1 & ses=3" to compare this cell to the "To1" cell. Do not use with meth=corr;
* To9=no         : When tests are required, compare1 is tested against the cell specified by the To1 parameter. For
*                  example, "To1 = itsex=2 & ses=3" . Do not use with meth=corr. You can request up to 9 tests;
* label9=-       : When tests are required, this parameter gives the name you want to give to your test;
* stat=mean      : To use with method=MEAN or method=PERC - Use stat=n where 1<=n<=99 with PERC to get estimated quantiles);
* where=         : To use with method=logistic & Multinomial - identifies which levels of dvar has to be kept in the regression);
*                : For example, <where= 1 2> indicates that only level 1 and 2 of dvar will be considered );
* fay=0.7        : Fay coefficient for studies using the BRR method to compute the variance (0.7 by default);
* ddf=           : The number of degrees of freedom. If left unspecified then stattool will generate the appropriate value);
* clev= .975     : The confidence limit level (.975 by default). To be used with method=PERC only ;
* JKZ= jkzone    : The zone assignment (JKZONE by default, only when study = TIMSS or PIRLS);
* JKI= jkrep     : Whether to zero or double the weights (1 or 0 only - JKREP by default,
                  only when study = TIMSS or PIRLS) ;
* out=c:\temp\out : Any name to be given to the html output file. Complete path has to be specified without quotes.
                  For example out=c:\temp\output1 will put the results in the file "c:\temp\output1.html"
                  By default, the results go into "C:\temp\out.html";
* items=         : Average Percent Correct only: Put all Items  if items have been already scored. Use xright parameters otherwise ;
* pts_2 =        : Average Percent Correct only: Put the items worth a maximum of 2 points. To be used only when scored=1;
* aright=        : Average Percent Correct only: Put all Items for which choice A was the correct answer ;
* bright=        : Average Percent Correct only: Put all Items for which choice B was the correct answer ;
* cright=        : Average Percent Correct only: Put all Items for which choice C was the correct answer ;
* dright=        : Average Percent Correct only: Put all Items for which choice D was the correct answer ;
* eright=        : Average Percent Correct only: Put all Items for which choice E was the correct answer ;
* o1right=       : Average Percent Correct only: Put all open-ended Items worth 1 point ;
* o2right=       : Average Percent Correct only: Put all open-ended Items worth 2 points;
* scored=0       : Indicator that tells us whether the items were already scored or not. By default, scored = 0 ;
* PCT_correct=0  : Average Percent Correct only: Put 1 if you want Average Percent Correct to be computed;
* PCT_cond=      : Average Percent Correct only: Put the condition that will restrict the data base to the records of interest;
*                : For example, put <pct_cond=if intms03 & intms99> to get Average Percent Correct for the trend items  ;
* international= : Put <international = 1> to get the international line. Will work if only one byvar variable) ;
* criter1=       : Any SAS statement(can be used to program an IF statement) ;
* criter2=       : Any SAS statement(can be used to program an IF statement) ;
* criter3=       : Any SAS statement(can be used to program an IF statement) ;
* pctlbrk=       : Any breakdown variable (No PVs) for the percentage of students over a given quartile) ;
*                : To be used only with meth=perc with no ivar. You then run a method=mean with dvar=PCTL and infile=all91z) ;

* First count the number of independent variables and create vector with
  the constant, the independent variable, and dependent variable;

%if &pct_correct = 1 %then %do;%let method = mean; %end;

%if %upcase(&study)=ALL %then %do;%let c=((&nrep-1) / &nrep);
                                  %if %length(%scan(&rwgt,1))=0 %then %let rwgt = replic ;
                                  %if %length(%scan(&ddf,1))=0 %then %let df=(&nrep-1);%else %let df=&ddf;
                              %end;
%else %if %upcase(&study)=TIMSS | %upcase(&study)=PIRLS %then %do; %let c=1;
                                                                   %let rwgt = replic ;
                                                                   %let cntryno = idcntry;
                                                                   %if %length(%scan(&ddf,1))=0 %then %let df=&nrep;%else %let df=&ddf;
                                                              %end;
%else %if %upcase(&study)=PISA %then %do; %let c=(1/(&nrep*(1-&fay)*(1-&fay)));
                                          %if %length(%scan(&ddf,1))=0 %then %let df=&nrep;%else %let df=&ddf;
                                     %end;
%else %if %upcase(&study)=OTHERS %then %do; 
                                          %if %length(%scan(&ddf,1))=0 %then %let df=&nrep;%else %let df=&ddf;
                                       %end;

/***************************************For PCT Correct Only***************************************************************************/

%if &pct_correct = 1 %then %do;

   %macro scoreit (item,type,right,nr,na,om,other);                                                                                        
    %if %upcase (&type) = "MC" %then %do;                                                                                                  
        score = 0;                                                                                                                         
          if &item = &right then score = 1;else                                                                                            
          if &item = &nr    then score = 0;else                                                                                            
          if &item = &na    then score = .;else                                                                                            
          if &item = &om    then score = 0;else                                                                                            
          if &item = &other then score = 0;                                                                                                
        &item = score;                                                                                                                     
       /* if score ne . then do;max+1;ind_score+score;end;  */                                                                                 
                                     %end;%else                                                                                            
    %if %upcase(&type) = "O1" %then %do;                                                                                                   
        score = 0;                                                                                                                         
          if (&item >=10 & &item<=19) then score = 1;else                                                                                  
          if &item  = &nr             then score = 0;else                                                                                  
          if (&item >=70 & &item<=79) then score = 0;else                                                                                  
          if &item = &na              then score = .;else                                                                                  
          if &item = &om              then score = 0;                                                                                      
        &item = score;                                                                                                                     
        /* if score ne . then do;max+1;ind_score+score;end;  */                                                                                 
                                   %end;%else                                                                                              
    %if %upcase(&type) = "O2" %then %do;                                                                                                   
        score = 0;                                                                                                                         
          if (&item >=20 & &item<=29) then score = 2;else                                                                                  
          if (&item >=10 & &item<=19) then score = 1;else                                                                                  
          if &item  = &nr             then score = 0;else                                                                                  
          if (&item >=70 & &item<=79) then score = 0;else                                                                                  
          if &item = &na              then score = .;else                                                                                  
          if &item = &om              then score = 0;                                                                                      
        &item = score;                                                                                                                     
       /*  if score ne . then do;max+2;ind_score+score;end;     */                                                                              
                                   %end;                                                                                                   
   %mend scoreit;                                                                                                                          
    
   %if %length(%scan(&pts_2,1))=0 %then %let pts_2=&o2right;
                                                                                                                                   
   %if %length(%scan(&items,1))=0 %then %let items = &aright &bright &cright &dright &eright &o1right &o2right ;
                                                                                  
           %end;   
/*************************************************************************************************************************************/

%if %lowcase(&method) =regr %then %do; %let proc=glm;  %end;
%else %if %lowcase(&method) = corr | %lowcase(&method) = corrdiff %then %do; %let proc=corr;
                                                                             %let ivar = &dvar &ivar;
                                                                             %let ivarpv = &dvarpv &ivarpv;
                                                                             %let dvar= ;
                                                                             %let dvarpv= ;
                                                                        %end;
%else %if %lowcase(&method) = diff %then %do; %let proc=glm; %let clvar=&ivar; %let clvarpv=&ivarpv; %end;
%else %if %lowcase(&method) =mean | %lowcase(&method) =popest | %lowcase(&method) = stddev %then %do;
                                             %let proc=means;
                                             %if %lowcase(&method) = stddev %then %do;
                                                                                      %let vardef=wdf;
                                                                                      %let stat=std;
                                                                                  %end;
                                             %else %do;%let vardef=df;%end;
                                             %if %length(%scan(&ivar,1))=0 %then %let ivar=domain;
                                             %let clvar=&ivar;
                                             %let clvarpv=&ivarpv;
                                             %if %lowcase(&method)=popest %then %do; %let dvar= ;
                                                                                     %let dvarpv= ;
                                                                                     %let stat=sum;
                                                                                %end;
                                                                                                 %end;
%else %if %lowcase(&method) =perc %then %do; %let proc=univariate;
                                             %if %length(%scan(&ivar,1))=0 %then %let ivar=domain;
                                             %let clvar=&ivar; %let clvarpv=&ivarpv;
                                        %end;
%else %if %lowcase(&method) =crosstabs %then %do; %let proc=freq;
                                                  %let clvar=;
                                                  %let clvarpv=;
                                                  %let ivar = &dvar &ivar;
                                                  %let ivarpv = &dvarpv &ivarpv;
                                                  %if %length(%scan(&ivar,3))>0 %then %do;
                                                                                          %let ivar = %scan(&ivar,1) %scan(&ivar,2) ;
                                                                                          %let ivarpv = %scan(&ivarpv,1) %scan(&ivarpv,2) ;
                                                                                     %end; 
                                                  %let dvar= ;
                                                  %let dvarpv= ;
                                                  %end;
%else %if %lowcase(&method) =multinomial %then %do; %let proc=logistic;%let link=glogit; %end;
%else %if %lowcase(&method) =survival %then %do; %let proc=logistic;%let link=cloglog; %end;
%else %if %lowcase(&method) =probit %then %do; %let proc=logistic;%let link=probit; %end;
%else %if %lowcase(&method) =logistic %then %do; %let proc=logistic;%let link=logit; %end;

%let byflag=0;
%let dflag=0;
%let clflag=0;
%let quflag=0;
%let iflag=0;
%let domain=ALL;
%let i = 1;                            * Count the option;
%do %while(%length(%scan(&option,&i)));
    %let i = %eval(&i + 1);
%end;
%let noption = %eval(&i-1);
%let i = 1;                            * Count the event;
%do %while(%length(%scan(&event,&i)));
    %let i = %eval(&i + 1);
%end;
%let nevent = %eval(&i-1);

%let i = 1;                            * Count the Where;
%do %while(%length(%scan(&where,&i)));
    %let i = %eval(&i + 1);
%end;
%let nwhere = %eval(&i-1);

%let i = 1;                            * Count the Interaction terms in a regression model;
%do %while(%length(%scan(&interaction,&i)));
    %let i = %eval(&i + 1);
%end;
%let nterm = %eval(&i-1);

%let i = 1;                            * Count the Byvars;
%do %while(%length(%scan(&byvar,&i)));
    %let i = %eval(&i + 1);
%end;
%let nbyvar = %eval(&i-1);
%let realnby = &nbyvar;
%if &nbyvar=0 %then %do;%let nbyvar=1; %let byvarpv=0; %end;
%do i=1 %to &nbyvar;
    %if %scan(&byvarpv,&i)=1 %then %do;
                                       %let byflag = %eval(&i+1);
                                   %end;
%end;
%let nquant = %eval(&i-1);
%let i = 1;                            * Count the Quant;
%do %while(%length(%scan(&quant,&i)));
    %let i = %eval(&i + 1);
%end;
%let nquant = %eval(&i-1);
%do i=1 %to &nquant;
    %if %scan(&quantpv,&i)=1 %then %do;
                                       %let quflag = %eval(&i+1);
                                   %end;
%end;
%let i = 1;                            * Count the CLvars;
%do %while(%length(%scan(&clvar,&i)));
    %let i = %eval(&i + 1);
%end;
%let nclvar = %eval(&i-1);
%do i=1 %to &nclvar;
    %if %scan(&clvarpv,&i)=1 %then %do;
                                       %let clflag = %eval(&i+1);
                                   %end;
%end;
%let mnvar=%eval(&realnby + &nclvar);
%let i = 1;                            * Count the Dvars;
%do %while(%length(%scan(&dvar,&i)));
    %let i = %eval(&i + 1);
%end;
%let ndvar = %eval(&i-1);
%do i=1 %to &ndvar;
    %if %scan(&dvarpv,&i)=1 %then %do;
                                       %let dflag = %eval(&i+1);
                                   %end;
%end;
%let i = 1;                            * Count the Ivars;
%do %while(%length(%scan(&ivar,&i)));
    %let i = %eval(&i + 1);
%end;
%let nivar = %eval(&i-1);
%do i=1 %to &nivar;
    %if %scan(&ivarpv,&i)=1 %then %do;
                                       %let iflag = %eval(&i+1);
                                   %end;
%end;
%if &nivar>0 %then %do;
%let lastivar = %scan(&ivar,&nivar);   * Last IVAR;
                   %end;

/* file prep for testing differences  */

%if  (%lowcase(&compare1) =no) | (%lowcase(&compare1) ~=no  && &proc=corr)  %then %do;
     %if %lowcase(&compare1) ~=no  && &proc=corr %then %let method=corrdiff;
    
* Read the data file and keep only what is necessary sorting by the
  subpopulation variables (ByVAR);
%if %lowcase(&infile) ~= setup %then %do;
data setup (keep= un iiddno Domain &wgt &byvar &ivar &dvar &cntryno &pctlbrk
     %if &PCT_correct = 1 %then %do; &items  %end;
     %if &nrep<    10 %then %do; &rwgt.1 - &rwgt&nrep %end; %else
     %if &nrep<   100 %then %do; &rwgt.01 - &rwgt&nrep %end; %else
     %if &nrep<  1000 %then %do; &rwgt.001 - &rwgt&nrep %end; %else
     %if &nrep< 10000 %then %do; &rwgt.0001 - &rwgt&nrep %end;
     %let i = 1;
     %do %while(%length(%scan(&ivar,&i)));
         %if %scan(&ivarpv, &i) =1 %then %do; %scan(&ivar, &i)1 - %scan(&ivar, &i)&npv %end;
         %let i = %eval(&i + 1);
     %end;
     %let i = 1;
     %do %while(%length(%scan(&byvar,&i)));
         %if %scan(&byvarpv, &i) =1 %then %do; %scan(&byvar, &i)1 - %scan(&byvar, &i)&npv %end;
         %let i = %eval(&i + 1);
     %end;
     %let i = 1;
     %do %while(%length(%scan(&dvar,&i)));
         %if %scan(&dvarpv, &i) =1 %then %do; %scan(&dvar, &i)1 - %scan(&dvar, &i)&npv %end;
         %let i = %eval(&i + 1);
     %end;
                         );
     idd+1;
     un=1;
set &infile;
     Domain="&domain";
     %if (%length(%scan(&byvar,1)))=0 %then %do;%let byvar=domain; %end;
     %if (%length(%scan(&ivar,1)))=0 %then %do;%let ivar=domain; %end;
     %if (%length(%scan(&dvar,1)))=0 %then %do;%let dvar=domain; %end;
     %let i = 1;
     %do %while(%length(%scan(&byvar,&i)));
         %if %scan(&byvarpv, &i) =1 %then %do; %scan(&byvar, &i)=%scan(&byvar,&i)1;;
                                          %end;
         %let i = %eval(&i + 1);
     %end;
             ;
     %let i = 1;
     %do %while(%length(%scan(&dvar,&i)));
         %if %scan(&dvarpv, &i) =1 %then %do; %scan(&dvar, &i)=%scan(&dvar,&i)1;;
                                          %end;
         %let i = %eval(&i + 1);
     %end;
             ;
     %let i = 1;
     %do %while(%length(%scan(&ivar,&i)));
         %if %scan(&ivarpv, &i) =1 %then %do; %scan(&ivar, &i)=%scan(&ivar,&i)1;;
                                          %end;
         %let i = %eval(&i + 1);
     %end;
             ;
     %let i = 1;
     %do %while(%length(%scan(&clvar,&i)));
         %if %scan(&ref,1) = default %then %do; %end;
         %else %do;
         if %scan(&clvar,&i) = %scan(&ref,&i) then %scan(&clvar,&i) = 9999;
               %end;
         %let i = %eval(&i + 1);
     %end;
     %if %upcase(&study)=TIMSS | %upcase(&study)=PIRLS %then %do;
         array rwgt  &rwgt.01  - &rwgt&nrep  ;         * Replicate Weight;
         * Computes the replicate weights;
         do i=1 to &nrep;
            if       &jkz ~= i             then rwgt(i) = &wgt * 1 ;
            else if (&jkz  = i & &jki = 1) then rwgt(i) = &wgt * 2 ;
            else if (&jkz  = i & &jki = 0) then rwgt(i) = &wgt * 0 ;
         end;
                                                             %end;
          
     iiddno +1;
/*********************************************************************************************************************************/

     %if &pct_correct = 1 %then %do;
     %if &scored = 0 %then %do;
     %if %length(%scan(&aright,1)) > 0 %then %do;array aright &aright; do over aright;%scoreit(aright,"MC",1, .r, .A, ., .i);end; %end;
     %if %length(%scan(&bright,1)) > 0 %then %do;array bright &bright; do over bright;%scoreit(bright,"MC",2, .r, .A, ., .i);end; %end;
     %if %length(%scan(&cright,1)) > 0 %then %do;array cright &cright; do over cright;%scoreit(cright,"MC",3, .r, .A, ., .i);end; %end; 
     %if %length(%scan(&dright,1)) > 0 %then %do;array dright &dright; do over dright;%scoreit(dright,"MC",4, .r, .A, ., .i);end; %end;
     %if %length(%scan(&eright,1)) > 0 %then %do;array eright &eright; do over eright;%scoreit(eright,"MC",5, .r, .A, ., .i);end; %end;
     %if %length(%scan(&o1right,1)) > 0 %then %do;array o1right &o1right; do over o1right;%scoreit(o1right,"O1", , .r, .A, ., .i);end; %end;
     %if %length(%scan(&o2right,1)) > 0 %then %do;array o2right &o2right; do over o2right;%scoreit(o2right,"O2", , .r, .A, ., .i);end; %end;                                                                                                             
                                                                            
     /* if ind_score ne . & max ne 0 then ind_score = ind_score / max;                                                                                
                                                                                                                                 
     max=0;ind_score=.; */    
                          %end;                                                                                                            
   
     %let DVAR = score;
     %let DVARPV = 0;
                            %end;
/****************************************************************************************************************************************/                                                                                       
     
     where &wgt is not missing; 
     title1 "Study: %upcase(&study)   : &criter1 , &criter2, &criter3 ";
     &criter1;
     &criter2;
     &criter3;
     %if &pct_correct = 1 %then %do; &pct_cond ; %end;
run;


run;                                                                                                                                    
                         
proc sort data = setup
     out = a;
     by &byvar;
run;
                    %end;
* Finds out which analytical tool was requested and runs it;
%if &proc = univariate %then %do; %end;
%else %if &PCT_correct = 1 %then %do;

        %do j=1 %to &nrep;         
        proc summary data=a nway;class &byvar &ivar; var &items;weight  
              %if &nrep <    10 %then %let ai = %eval(&j); %else
              %if &nrep <   100 %then %do;
                  %if %eval(&j) < 10 %then %let ai=0&j; %else
                  %let ai = %eval(&j);
                                      %end; %else
              %if &nrep <  1000 %then %do;
                  %if %eval(&j) <  10 %then %let ai=00&j; %else
                  %if %eval(&j) < 100 %then %let ai=0&j; %else
                  %let ai = %eval(&j);
                                      %end; %else
              %if &nrep < 10000 %then %do;
                  %if %eval(&j) <   10 %then %let ai=000&j; %else
                  %if %eval(&j) <  100 %then %let ai=00&j; %else
                  %if %eval(&j) < 1000 %then %let ai=0&j; %else
                  %let ai = %eval(&j);
                                      %end;                                                                                         
              &rwgt&ai;
;
                                                                            output out=stat&j mean = 
                                                                                                   %let i=1;                                           
                                                                                                   %do %while(%length(%scan(&items,&i)));                         
                                                                                                       impu_%scan(&items,&i)
                                                                                                   %let i = %eval(&i+1);                    
                                                                                                   %end;
                                                                                               max =  
                                                                                                   %let i=1;                                           
                                                                                                   %do %while(%length(%scan(&items,&i)));                         
                                                                                                       max_%scan(&items,&i)
                                                                                                   %let i = %eval(&i+1);                    
                                                                                                   %end;;                                       
             run; 

                                                                                                                                        
           data stat&j;set stat&j;
                score = 0;
                maxpts = 0;
                %let i=1;                                           
                %do %while(%length(%scan(&items,&i)));                         
                    if impu_%scan(&items,&i) ne . then score + impu_%scan(&items,&i);
                    if max_%scan(&items,&i) ne . then maxpts+1;
                    %let k=1;
                    %do %while(%length(%scan(&pts_2,&k)));
                        %if %lowcase(%scan(&pts_2,&k)) = %lowcase(%scan(&items,&i)) %then %do;
                                                                          if max_%scan(&items,&i) ne . then maxpts+1;
                                                                                          %end;
                        %let k = %eval(&k+1);
                    %end;
                    %let i = %eval(&i+1);                    
                %end;                                                                                                                              
                estimate=100*score / maxpts;                                                                      
                drop %let i=1;%do %while(%length(%scan(&items,&i))); impu_%scan(&items,&i) %let i = %eval(&i+1);%end; score maxpts;                                                                                                             
         %end;
         
        proc summary data=a nway;class &byvar &ivar; var &items;weight &wgt;  output out=impu1 mean = 
                                                                                                   %let i=1;                                           
                                                                                                   %do %while(%length(%scan(&items,&i)));                         
                                                                                                       impu_%scan(&items,&i)
                                                                                                   %let i = %eval(&i+1);                    
                                                                                                   %end;
                                                                                               max =  
                                                                                                   %let i=1;                                           
                                                                                                   %do %while(%length(%scan(&items,&i)));                         
                                                                                                       max_%scan(&items,&i)
                                                                                                   %let i = %eval(&i+1);                    
                                                                                                   %end;;                                       
             run; 

           data impu1;set impu1;
                score = 0;                                                                                                   
                maxpts = 0;
                %let i=1;                                           
                %do %while(%length(%scan(&items,&i)));                         
                    if impu_%scan(&items,&i) ne . then score + impu_%scan(&items,&i);
                    if max_%scan(&items,&i) ne . then maxpts+1;
                    %let k=1;
                    %do %while(%length(%scan(&pts_2,&k)));
                        %if %lowcase(%scan(&pts_2,&k)) = %lowcase(%scan(&items,&i)) %then %do;
                                                                          if max_%scan(&items,&i) ne . then maxpts+1;
                                                                                          %end;
                        %let k = %eval(&k+1);
                    %end;
                    %let i = %eval(&i+1);                    
                %end;                                                                                                                              
                estimate=100*score / maxpts;                                                                      
                drop %let i=1;%do %while(%length(%scan(&items,&i))); impu_%scan(&items,&i) %let i = %eval(&i+1);%end; score maxpts;                                                                                                             
                                                                                    
                                  %end;                                                                                                      
   

%else %do;
    %do lp=1 %to &nrep;
             
           
        ods listing close;
        %if &proc = glm %then %do; ods output "Solution"=stat&lp; %end;
        %else %if &proc = freq %then %do; ods output "Cross-Tabular Freq Table"=stat&lp; %end;
        %else %if &proc = catmod %then %do; ods output "ML Estimates"=stat&lp; %end;
        %else %if &proc = logistic %then %do; ods output "Parameter estimates"=stat&lp; %end;

        %if &proc = corr %then %do;proc corr data=a noprint outp=stat&lp;%end;
        %else %do;proc &proc data=a  %if %lowcase(&method) = stddev %then %do; vardef=&vardef %end;;%end;
        by &byvar;
        %if &nclvar = 0 %then %do;%end;%else %do; class &clvar %if &proc = logistic %then %do; / param=glm; %end;%else %do; ; %end;%end;  
        /*
        %if &nquant = 0 %then %do;%end;%else %do; direct &quant; %end;
        */
        %if &proc = catmod %then %do; direct &ivar; %end;
        %if %lowcase(&method) = regr %then %do; model &dvar = %if &nivar>0 %then %do; &ivar %end;  
                                                              %if &nterm>0 %then %do; &interaction %end; / 
                                                              %if &noption>0 %then %do; &option %end; solution ; %end;
        %else %if %lowcase(&method) = corr | %lowcase(&method) = corrdiff %then %do; var &ivar; %end;
        %else %if %lowcase(&method) = diff %then %do;
                                                     model &dvar =
                                                     %do i=1 %to &nivar;
                                                         %scan(&ivar,&i)
                                                         %if &i=&nivar %then %do; / solution; %end;
                                                         %else %do; * %end;
                                                     %end;
                                                 %end;
        %else %if &proc = catmod %then %do;
                                           %if &nwhere>0 %then %do; where &dvar in (&where); %end;
                                           model &dvar = %if &nivar>0 %then %do; &ivar %end;  
                                                         %if &nterm>0 %then %do; &interaction %end;  / 
                                                         %if &noption>0 %then %do; &option %end;  noiter ;
                                           %if %lowcase(&method)=adjacent %then %do; response &type; %end;
                                       %end;
        %else %if &proc = logistic %then %do;
                                           %if &nwhere>0 %then %do; where &dvar in (&where); %end;
                                           model &dvar %if &nevent>0 %then %do;(event="&event") %end; = 
                                                         %if &nivar>0 %then %do; &ivar %end;  
                                                         %if &nterm>0 %then %do; &interaction %end;  / 
                                                         %if &noption>0 %then %do; &option %end;  link=&link ;
                                         %end;
        %else %if &proc = means %then %do; %if %lowcase(&stat)=sum %then %do; var un; %end;
                                           %else %do; var &dvar; %end;
                                      %end;
        %else %if &proc = freq %then %do; tables
                                           %do i=1 %to &nivar;
                                           %scan(&ivar,&i)
                                           %if &nivar=1 %then %do; *un %end;
                                           %if &i=&nivar %then %do;; %end; %else %do; * %end;
                                           %end;;
                                     %end;
        weight
              %if &nrep <    10 %then %let ai = %eval(&lp); %else
              %if &nrep <   100 %then %do;
                  %if %eval(&lp) < 10 %then %let ai=0&lp; %else
                  %let ai = %eval(&lp);
                                      %end; %else
              %if &nrep <  1000 %then %do;
                  %if %eval(&lp) <  10 %then %let ai=00&lp; %else
                  %if %eval(&lp) < 100 %then %let ai=0&lp; %else
                  %let ai = %eval(&lp);
                                      %end; %else
              %if &nrep < 10000 %then %do;
                  %if %eval(&lp) <   10 %then %let ai=000&lp; %else
                  %if %eval(&lp) <  100 %then %let ai=00&lp; %else
                  %if %eval(&lp) < 1000 %then %let ai=0&lp; %else
                  %let ai = %eval(&lp);
                                      %end;
              &rwgt&ai;
        %if &proc = means %then %do; %if &nivar=1 %then %do; ways  1; %end; %else %do; ways &nclvar; %end;
                                     output out=stat&lp &stat=estimate;
                                %end;
        quit;
        ods output close;
        ods listing;
    %end;
      %end;

   %if &PCT_correct = 1 %then %do;%end;

   %else %do lp=1 %to &npv;
        %if &lp>1 %then %do;
           %if &byflag>0 | &clflag>0 | &quflag>0 | &iflag>0 | &dflag >0 %then %do;
              data a;set a;
                 %if &lp = 2 %then %do;
                 Drop
                 %if &nrep<    10 %then %do; &rwgt.1 - &rwgt&nrep %end; %else
                 %if &nrep<   100 %then %do; &rwgt.01 - &rwgt&nrep %end; %else
                 %if &nrep<  1000 %then %do; &rwgt.001 - &rwgt&nrep %end; %else
                 %if &nrep< 10000 %then %do; &rwgt.0001 - &rwgt&nrep %end;
                     ;
                                   %end;
                 %if &byflag>0 %then %do;
                 %let i = 1;
                      %do %while(%length(%scan(&byvar,&i)));
                          %if %scan(&byvarpv, &i) =1 %then %do; %scan(&byvar, &i)=%scan(&byvar,&i)&lp;
                                                           %end;
                          %let i = %eval(&i + 1);
                      %end;
                ;
                                     %end;
                 %if &dflag>0 %then %do;
                 %let i = 1;
                      %do %while(%length(%scan(&dvar,&i)));
                          %if %scan(&dvarpv, &i) =1 %then %do; %scan(&dvar, &i)=%scan(&dvar,&i)&lp;
                                                           %end;
                          %let i = %eval(&i + 1);
                      %end;
                                     %end;
                ;
                 %if &iflag>0 %then %do;
                 %let i = 1;
                      %do %while(%length(%scan(&ivar,&i)));
                          %if %scan(&ivarpv, &i) =1 %then %do; %scan(&ivar, &i)=%scan(&ivar,&i)&lp;
                                                           %end;
                          %let i = %eval(&i + 1);
                      %end;
                ;
                                    %end;
         
                
                %let i = 1;
                    %do %while(%length(%scan(&clvar,&i)));
                        %if %scan(&ref,1) = default %then %do; %end;
                    %else %do;
                        if %scan(&clvar,&i) = %scan(&ref,&i) then %scan(&clvar,&i) = 9999;
                          %end;
                   %let i = %eval(&i + 1);
                   %end;
                                   

           %if &byflag>0 %then %do;
             proc sort data=a;
             by &byvar ;
             quit;
                              %end;
                                                                           %end;
                       %end;
        ods listing close;
        %if &proc = glm %then %do;
                                  ods output "fit statistics"=fit&lp;
                                  ods output "number of observations"=obs&lp;
                                  ods output "Solution"=impu&lp;
                              %end;
        %else %if &proc = catmod %then %do;
                                  /* ods output "population profiles"=popu&lp; */
                                  ods output "response profiles"=resp&lp;
                                  ods output "Ml Estimates"=impu&lp;
                                      %end;
        %else %if &proc = logistic %then %do;
                                  ods output "response profile"=resp&lp;
                                  ods output "fit statistics"=info&lp;
                                  ods output "Parameter Estimates"=impu&lp;
                                      %end;
        %else %if &proc = freq %then %do; ods output "Cross-Tabular Freq Table"=impu&lp; %end;
        %if &proc = means %then %do; %end;
        %else %do;
              %if &proc = logistic %then %do; ods output "class level information"=lev&lp;
                                       %end;
              %else %do;
                        %if &nclvar=0 %then %do;%end;%else %do;
                                                               ods output "class levels"=lev&lp;
                                                            %end;
                    %end;
              %end;
        %if &proc = corr %then %do;proc corr data=a noprint outp=impu&lp;%end;
        %else %do;proc &proc data=a %if %lowcase(&method) = stddev %then %do;vardef=&vardef%end;;%end;

        by &byvar;
        %if &nclvar = 0 %then %do;%end;%else %do; class &clvar %if &proc = logistic %then %do; / param=glm; %end;%else %do; ; %end;%end;   
        %if &proc = catmod %then %do; direct &IVAR; %end;
        %if %lowcase(&method) = regr %then %do; model &dvar = %if &nivar>0 %then %do; &ivar %end;  
                                                              %if &nterm>0 %then %do; &interaction %end;  / 
                                                              %if &noption>0 %then %do; &option %end;  solution ; %end;
        %else %if %lowcase(&method) = corr | %lowcase(&method) = corrdiff  %then %do; var &ivar; %end;
        %else %if %lowcase(&method) = diff %then %do;
                                                     model &dvar =
                                                     %do i=1 %to &nivar;
                                                         %scan(&ivar,&i)
                                                         %if &i=&nivar %then %do; / solution; %end;
                                                         %else %do; * %end;
                                                     %end;
                                                 %end;
        %else %if &proc = catmod %then %do;
                                           %if &nwhere>0 %then %do; where &dvar in (&where); %end;
                                           model &dvar = %if &nivar>0 %then %do; &ivar %end;  
                                                         %if &nterm>0 %then %do; &interaction %end;  / 
                                                         %if &noption>0 %then %do; &option %end;  noiter ;
                                           %if %lowcase(&method)=adjacent %then %do; response &type; %end;
                                       %end;
        %else %if &proc = logistic %then %do;
                                           %if &nwhere>0 %then %do; where &dvar in (&where); %end;
                                           model &dvar %if &nevent>0 %then %do;(event="&event") %end; = 
                                                         %if &nivar>0 %then %do; &ivar %end;  
                                                         %if &nterm>0 %then %do; &interaction %end;  / 
                                                         %if &noption>0 %then %do; &option %end;  link=&link ;
                                         %end;
        %else %if &proc = means | &proc = univariate %then %do; %if %lowcase(&stat)=sum %then %do; var un; %end;
                                                                %else %do; var &dvar; %end;
                                                           %end;
        %else %if &proc = freq %then %do; tables
                                           %do i=1 %to &nivar;
                                           %scan(&ivar,&i)
                                           %if &nivar=1 %then %do; *un %end;
                                           %if &i=&nivar %then %do;; %end; %else %do; *  %end;
                                           %end;;


                               %end;
        weight &wgt;
        %if &proc = means %then %do; %if &nivar=1 %then %do; ways  1; %end; %else %do; ways &nclvar; %end;
                                     output out=impu&lp &stat=estimate;
                                %end;
        %else %if &proc = univariate %then %do; output out=impu&lp pctlpre=P_ pctlpts=&stat; %end;
        quit;
        %if &proc = glm %then %do; ods output close; ods output close; ods output close; %end;
        %if &proc = logistic %then %do; ods output close; ods output close; ods output close;ods output close; %end;
        %if &proc = catmod %then %do; ods output close; ods output close; /* ods output close; */ %end;
        %if &proc = freq %then %do; ods output close; %end;
        %if &proc = logistic %then %do; %end; %else %do;
                                                      %if &nclvar=0 %then %do;%end;
                                                      %else %do;ods output close; %end;
                                                     %end;
        
        ods listing;
        %if &proc = univariate %then %do;
        proc sort data=a;by &byvar &clvar;run;
        data imp&lp;merge impu&lp a;by &byvar &clvar;
             pctl&lp = (&dvar < P_&stat);
             keep iiddno pctl&lp;
        run;
        proc sort data=imp&lp;by iiddno;run;
                                     %end;
   %end;
   %if &proc = univariate %then %do;
                                     proc sort data=setup;by iiddno;run;
                                     data all91z;merge setup
                                     %do i=1 %to &npv;
                                         imp&i
                                     %end;
                                     ;
                                     by iiddno;
                                     run;
                                %end;
   %else %do;
* Computes sampling variance based on first plausible values;
*    Computes the jackknife repliclates;
   data all_rep;
        set
           %do i=1 %to &nrep;
               stat&i
           %end;
           ;
         %if &proc = freq && &nivar=1 %then %do; if _type_ in (1,11);%end;
         %if &proc = corr   %then %do;
                                      if _TYPE_ = 'STD' then delete;
                                      length parameter $ 32;
                                      parameter=_type_||_name_;
                                      drop _type_ _name_;
                                  %end;

         %if &proc = catmod %then %do;
                                      param=functionnumber||parameter;
                                                                drop parameter;
                                                                rename param=parameter;
                                                             
                                  %end;
         %if &proc = logistic %then %do; 
                                        %if &nwhere > 2 %then %do;
                                                                %if %lowcase(&method)~= multinomial %then %do;
                                                                    %let prefix=0;
                                                                    param=variable %do i=1 %to &nclvar;%let j=%eval(&i-1);
                                                                                                       ||" "||compress(classval&j) 
                                                                                   %end;;
                                                                    drop variable;
                                                                    param = scan(param,1,'.');
                                                                    rename param=parameter;                   
                                                                                                         %end;
                                                                %else  %do;
                                                                %let prefix=1;
                                                                %if &nclvar>0 %then %do;
                                                                    param=response||variable %do i=1 %to &nclvar;%let j=%eval(&i-1);
                                                                                                        ||" "||compress(classval&j) 
                                                                                             %end;;
                                                                    drop variable;
                                                                    param = scan(param,1,'.');
                                                                    rename param=parameter;                   
                                                                                    %end;
                                                                %else               %do;
                                                                    param=response||variable;
                                                                    drop variable;
                                                                    rename param=parameter;
                                                                                    %end;
                                                                       %end;
                                                            %end;
                                       %else                   %do;%let prefix=0;
                                                                   %if &nclvar>0 %then %do;
                                                                       param=variable %do i=1 %to &nclvar;%let j=%eval(&i-1);
                                                                                                          ||" "||compress(classval&j) 
                                                                                      %end;;
                                                                       drop variable;
                                                                       param = scan(param,1,'.');
                                                                       rename param=parameter;                   
                                                                                    %end;
                                                                   %else               %do;
                                                                       rename variable=parameter;
                                                                                       %end;
                                                              %end;

                                     %end;
   run;

   proc sort data=all_rep;by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;;run;
   
   proc summary data=all_rep nway;
        by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;;
        %if &proc = glm | &proc = catmod | &proc = logistic %then %do; class parameter;
                                                                     var estimate;
                                                                %end;
        %else %if &proc = corr %then %do;class parameter;var &ivar;%end;
        %else %if &proc = means %then %do; var estimate; %end;
        %else %if &proc = freq %then %do; var percent frequency %if &nivar > 1 %then %do; rowpercent colpercent %end;;
                                     %end;
        %if &proc = corr %then %do;
                                   output out=tot_rep css=css %do i=2 %to &nivar; css&i %end;
                                                      mean=ybar %do i=2 %to &nivar; ybar&i %end;;
                               %end;
        %else %do;
        output out=tot_rep css=css %if &proc = freq %then %do; css2 %if &nivar>1 %then %do;css3 css4 %end;%end;
                           mean=ybar %if &proc = freq %then %do; ybar2 %if &nivar>1 %then %do;ybar3 ybar4 %end;%end;;
              %end;
   run;

*    Computes the overall mean based on &wgt ;
   %do lp=1 %to &npv;
       data impu&lp;set impu&lp;if _n_=1 then id+1+&lp*100000;else id+1;
       
       %if &proc = freq && &nivar=1 %then %do; if _type_ in (1,11);%end;
       %if &proc=freq %then %do; drop _type_; %end;
       %if &proc = catmod %then %do;
                                      param=functionnumber||parameter;
                                                                drop parameter;
                                                                rename param=parameter;
                                      
                                %end;
       %if &proc = corr   %then %do;
                                      if _TYPE_ = 'STD' then delete;
                                      length parameter $ 32;
                                      parameter=_type_||_name_;
                                      drop _type_ _name_;
                                  %end;

       %if &proc = logistic %then %do;
                                      %if &nwhere > 2 %then %do;
                                                                %if %lowcase(&method)~= multinomial %then %do;
                                                                    %let prefix=0;
                                                                    param=variable %do i=1 %to &nclvar;%let j=%eval(&i-1);
                                                                                                       ||" "||compress(classval&j) 
                                                                                   %end;;
                                                                    drop variable;
                                                                    param = scan(param,1,'.');
                                                                    rename param=parameter;                   
                                                                                                         %end;
                                                                %else  %do;
                                                                %let prefix=1;
                                                                %if &nclvar>0 %then %do;
                                                                    param=response||variable %do i=1 %to &nclvar;%let j=%eval(&i-1);
                                                                                                                 ||" "||compress(classval&j) 
                                                                                             %end;;
                                                                    drop variable;
                                                                    param = scan(param,1,'.');
                                                                    rename param=parameter;                   
                                                                                    %end;
                                                                %else               %do;
                                                                    param=response||variable;
                                                                    drop variable;
                                                                    rename param=parameter;
                                                                                    %end;
                                                                       %end;
                                                            %end;
                                       %else                   %do;%let prefix=0;
                                                                   %if &nclvar>0 %then %do;
                                                                       param=variable %do i=1 %to &nclvar;%let j=%eval(&i-1);
                                                                                                          ||" "||compress(classval&j) 
                                                                                      %end;;
                                                                       drop variable;
                                                                       param = scan(param,1,'.');
                                                                       rename param=parameter;                   
                                                                                    %end;
                                                                   %else               %do;
                                                                       rename variable=parameter;
                                                                                       %end;
                                                              %end;

                                   %end;
       run;
   %end;
   proc sort data=impu1;by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;
                        %if &proc = glm | &proc = corr | &proc = catmod | &proc = logistic %then %do; parameter; %end;
   run;
*    Computes the sampling variances;
   data smp_var;merge impu1 tot_rep;by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;
                        %if &proc = glm | &proc = corr | &proc = catmod | &proc = logistic  %then %do; parameter; %end;
        %if &proc = freq %then %do;
        smp_var=&c*(css + &nrep*(ybar-percent)*(ybar-percent));
        smp_var2=&c*(css2 + &nrep*(ybar2-frequency)*(ybar2-frequency));
                                    %if &nivar > 1 %then %do;
        smp_var3=&c*(css3 + &nrep*(ybar3-rowpercent)*(ybar3-rowpercent));
        smp_var4=&c*(css4 + &nrep*(ybar4-colpercent)*(ybar4-colpercent));
                                                         %end;

                               %end;

        %else %if &proc = corr %then %do;
        smp_var=&c*(css + &nrep*(ybar- %scan(&ivar,1))*(ybar-%scan(&ivar,1)));
        %do i=2 %to &nivar;
        smp_var&i=&c*(css&i + &nrep*(ybar&i-%scan(&ivar,&i))*(ybar&i-%scan(&ivar,&i)));
        %end;
                                     %end;


        %else %do;
        smp_var=&c*(css + &nrep*(ybar-estimate)*(ybar-estimate));
              %end;
        keep &byvar id %if &proc = glm | &proc = corr | &proc = catmod | &proc = logistic  %then %do;
                                                                                                   parameter
                                                                                               %end;
                       %else %do; &ivar %end;
              smp_var %if &proc = freq %then %do; percent frequency smp_var2 %if &nivar > 1 %then %do;
                                                                                            smp_var3 smp_var4 rowpercent colpercent
                                                                                                  %end;;
                                             %end;
                     %else %if &proc = corr %then %do;&ivar  %do i=2 %to &nivar; smp_var&i  %end;;%end;
             %else %do; estimate ; %end;
   run;
* Computes the imputation variance based on all plausible values;
       %if &proc = glm %then %do;
       data all_fit;
            set
               %do i=1 %to &npv;
                   fit&i
               %end;
               ;
            drop cv rootmse;
       run;
       proc sort data=all_fit; by &byvar;run;
       proc summary data=all_fit nway;
            by &byvar ;
            var rsquare depmean ;
            output out=tot_fit mean=rsquare dep_mean;
       run;
                           %end;
   %if &npv>1 %then %do;
       data all_impu;
            set
               %do i=1 %to &npv;
                   impu&i
               %end;
               ;
       run;
       proc sort data=all_impu;by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;
                        %if &proc = glm | &proc = corr | &proc = catmod | &proc = logistic  %then %do; parameter; %end;run;

       %if &proc = freq && &npv > 1 %then %do;
       data all_impu (drop=comptr) ;set all_impu;
            by &byvar &ivar;
            output;
            comptr +1;
            if last.&lastivar then do;
               do while (comptr < &npv);percent=0; frequency=0; %if &nivar > 1 %then %do; rowpercent=.; colpercent=.; %end;
                                        output;comptr+1;
               end;
               comptr=0;
                                   end;
       run;
                                      %end;
       

       
       proc summary data=all_impu nway;
            by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;;
        %if &proc = glm  | &proc = catmod | &proc = logistic  %then %do; class parameter;
                                                                       var estimate;
                                                                  %end;
        %else %if &proc = corr %then %do;class parameter;var &ivar;%end;
        %else %if &proc = means %then %do; var estimate; %end;
        %else %if &proc = freq %then %do; var percent frequency %if &nivar > 1 %then %do; rowpercent colpercent %end;;
                                     %end;
        %if &proc = corr %then %do;
                                   output out=tot_impu css=css %do i=2 %to &nivar; css&i %end;
                                                      mean=&ivar;
                               %end;
        %else %do;
        output out=tot_impu css=css %if &proc = freq %then %do; css2 %if &nivar>1 %then %do;css3 css4 %end;%end;
                           mean= %if &proc = freq %then %do; percent frequency %end; %else %do; estimate %end;
                                 %if &proc = freq && &nivar > 1 %then %do; rowpercent colpercent %end;
                                 ;
              %end;
   run;

       data impu_var;set tot_impu;
            impu_var = (&npv+1)*css/(&npv*(&npv-1));
            %if &proc = freq %then %do;
            impu_var2 = (&npv+1)*css2/(&npv*(&npv-1));
            %if &nivar>1 %then %do;
            impu_var3 = (&npv+1)*css3/(&npv*(&npv-1));
            impu_var4 = (&npv+1)*css4/(&npv*(&npv-1));
                               %end;
                                   %end;
            %if &proc = corr %then %do i=2 %to &nivar;
            impu_var&i = (&npv+1)*css&i/(&npv*(&npv-1));
                                   %end;

            keep &byvar %if &proc = means | &proc = corr | &proc = freq %then %do; &ivar %end;
                        %if &proc = glm | &proc = corr | &proc = catmod | &proc = logistic  %then %do; parameter %end;
                        impu_var
                        %if &proc = freq %then %do; percent frequency impu_var2 %end;
                        %else %if &proc = corr %then %do;%do i=2 %to &nivar;impu_var&i %end;%end;
                        %else %do; estimate %end;
                        %if &proc = freq &&  &nivar > 1 %then %do; impu_var3 impu_var4 rowpercent colpercent
                                               %end;;
       run;
* Computes the overall variance ;
       data tot_var;merge smp_var impu_var;by &byvar %if &proc = means | &proc = freq %then %do; &ivar; %end;
                                           %if &proc = glm  | &proc = corr | &proc = catmod | &proc = logistic  %then %do; parameter; %end;
            tot_var = smp_var + impu_var;
            if tot_var =0 then fm=0;else  
            fm = impu_var / tot_var;
            df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
            ddf = &df;
            df = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df = &ddf; %end;  
            tot_se  = sqrt(tot_var);
            %if &proc = freq %then %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(percent/tot_se),df));
                                       else prob_T=.;
                                       tot_var2 = smp_var2 + impu_var2;
                                       if tot_var2 =0 then fm=0;else
                                       fm = impu_var2 / tot_var2;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df2 = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df2 = &ddf; %end; 
                                       tot_se2  = sqrt(tot_var2);
                                       if tot_se2 ne 0 then prob_T2  = 2*(1-probt(abs(frequency/tot_se2),df2));
                                       else prob_T2=.;
                                       %if &nivar > 1 %then %do;
                                       tot_var3 = smp_var3 + impu_var3;
                                       if tot_var3 =0 then fm=0;else
                                       fm = impu_var3 / tot_var3;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df3 = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df3 = &ddf; %end; 
                                       tot_se3  = sqrt(tot_var3);
                                       if tot_se3 ne 0 then prob_T3  = 2*(1-probt(abs(rowpercent/tot_se3),df3));
                                       else prob_T3=.;
                                       tot_var4 = smp_var4 + impu_var4;
                                       if tot_var4 =0 then fm=0;else
                                       fm = impu_var4 / tot_var4;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df4 = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df4 = &ddf; %end; 
                                       tot_se4  = sqrt(tot_var4);
                                       if tot_se4 ne 0 then prob_T4  = 2*(1-probt(abs(colpercent/tot_se4),df4));
                                       else prob_T4=.;
                                                           %end;
                                   %end;
            %else %if &proc = corr %then %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(%scan(&ivar,1)/tot_se),df));
                                       else prob_T=.;
                                       %do i=2 %to &nivar;
                                       tot_var&i = smp_var&i + impu_var&i;
                                       if tot_var&i =0 then fm=0;else
                                       fm = impu_var&i / tot_var&i;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df&i = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df&i = &ddf; %end; 
                                       tot_se&i  = sqrt(tot_var&i);
                                       if tot_se&i ne 0 then prob_T&i  = 2*(1-probt(abs(%scan(&ivar,&i)/tot_se&i),df&i));
                                       else prob_T&i=.;
                                       %end;
                                         %end;

            %else %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),df));
                                       else prob_T=.;
                  %end;
       run;
                     %end;
    %else %do;
       
       data tot_var;set smp_var;
            tot_var = smp_var;
            df = &df;
            tot_se  = sqrt(tot_var);
            %if &proc = freq %then %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(percent/tot_se),&df));
                                       else prob_T=.;
                                       tot_var2 = smp_var2;
                                       df2 = &df;
                                       tot_se2  = sqrt(tot_var2);
                                       if tot_se2 ne 0 then prob_T2  = 2*(1-probt(abs(frequency/tot_se2),&df));
                                       else prob_T2=.;
                                       %if &nivar > 1 %then %do;
                                       tot_var3 = smp_var3;
                                       df3 = &df;
                                       tot_se3  = sqrt(tot_var3);
                                       if tot_se3 ne 0 then prob_T3  = 2*(1-probt(abs(rowpercent/tot_se3),&df));
                                       else prob_T3=.;
                                       tot_var4 = smp_var4;
                                       tot_se4  = sqrt(tot_var4);
                                       df4 = &df;
                                       if tot_se4 ne 0 then prob_T4  = 2*(1-probt(abs(colpercent/tot_se4),&df));
                                       else prob_T4=.;
                                                            %end;
                                   %end;
            %else %if &proc = corr %then %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(%scan(&ivar,1)/tot_se),&df));
                                       else prob_T=.;
                                       %do i=2 %to &nivar;
                                       tot_var&i = smp_var&i ;
                                       df&i = &df;
                                       tot_se&i  = sqrt(tot_var&i);
                                       if tot_se&i ne 0 then prob_T&i  = 2*(1-probt(abs(%scan(&ivar,&i)/tot_se&i),&df));
                                       else prob_T&i=.;
                                       %end;
                                          %end;

            %else %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),&df));
                                       else prob_T=.;
                  %end;
       run;
          %end;

    %if &international = 1 && &nbyvar = 1 && &byvar = &cntryno && &proc ~= freq && &proc ~= corr && 
                                                                             &proc ~= univariate && %lowcase(&method) ~= diff  %then %do;
        
        proc summary data=tot_var nway;class %if &proc = means %then %do;&ivar %end; %else %do; parameter %end;;
                     var estimate smp_var %if &npv=1 %then %do;%end;%else %do;impu_var %end;;
                     output out = international mean = estimate 
                                                n=n 
                                                sum= dummy smp_var  %if &npv=1 %then %do;%end;%else %do;impu_var %end;;  
        
        data international;set international;
             %if &npv > 1 %then %do;
             tot_var = (smp_var + impu_var) / (n*n);
             if tot_var =0 then fm=0;else  
             fm = (impu_var/(n*n)) / tot_var;
             df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
             ddf = &df;
             df = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df = &ddf; %end;  
             tot_se  = sqrt(tot_var);
             if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),df));
             else prob_T=.;
                                %end;
             %else %do;
             tot_var = smp_var/(n*n);
             df = &df;
             tot_se  = sqrt(tot_var);
             if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),&df));
             else prob_T=.;
                   %end;
             drop n dummy;

        data tot_var;set tot_var international;run;
                                                                                           %end;
        
    proc sort data=tot_var;by &byvar id; run;     

    %if &proc = logistic  && &nivar>0 %then %do;
        
       data all_info;
           set
               %do i=1 %to &npv;
                   info&i
               %end;
              ;
           if criterion = "-2 Log L";
       run;

      proc sort data=all_info;by &byvar;run;

      data all_info;set all_info;
           by &byvar;
           if first.&byvar then value=0;
           value+1;
           interceptonly=interceptonly/(-2);
           interceptandcovariates=interceptandcovariates/(-2);
           Negative=1-interceptandcovariates/interceptonly;
           keep &byvar value interceptonly interceptandcovariates Negative;
           label value                 ="Plausible Value"                                                                                        
                 interceptonly         ="Intercept Only"                                                                                         
                 interceptandcovariates="Intercept and Covariates"                                                                               
                 negative              ="Negative Log Likelihood"; 
       run;
                                            %end;

    data %if &proc = glm %then %do;
         finala(keep = &byvar dependent rsquare dep_mean n_obs %if &nclvar>0 %then %do; class levels values %end;)
                               %end;
         %if &proc = logistic && &nclvar>0 %then %do;
         finala
                                                 %end;

         finalb(keep = &byvar %if &proc = glm | &proc = corr | &proc = catmod | &proc = logistic %then %do; parameter %end; %else %do; &ivar %end;
                              %if &proc = catmod | &proc = logistic %then %do; odds_ratio lower upper %end;
                              %if &proc = freq %then %do; percent tot_se df prob_T frequency tot_se2 df2 prob_t2
                                                          %if &nivar > 1 %then %do;
                                                          rowpercent tot_se3 colpercent tot_se4 df3 df4 prob_t3 prob_t4
                                                                               %end;
                                                                             );
                                                     %end;
                              %else %if &proc = corr %then %do; %do;&ivar tot_se df prob_T %do i=2 %to &nivar;df&i tot_se&i %end; prob_T2-prob_T&nivar %end;);%end;
                              %else %do; estimate tot_se df prob_T); %end;
         merge
               %if &proc = means %then %do; %end;
               %else %do;
               %if &nclvar>0 %then %do; lev1 %end;
                     %end;
               %if &proc = glm %then %do;
                                         obs1 tot_fit tot_var;
                                     %end;
               %else %do; tot_var ; %end;
         by &byvar;
         %if &proc = glm %then %do;
                                    dependent="&dvar";
                                    n_obs = cvalue1;
                                    drop cvalue1;
                                    output finala;
                               %end;
         %if &proc = logistic && &nclvar>0 %then %do;output finala;%end;
         %if &proc = catmod | &proc = logistic %then %do;
             if /* substr(parameter,1,6)='Interc'  or
                   substr(parameter,2,6)='Interc'  or
                   substr(parameter,3,6)='Interc'  or */ tot_se=0 then do;odds_ratio=.;LOWER=.;UPPER=.;end;
             else do;odds_ratio = exp(estimate);LOWER=exp(estimate-tot_se*tinv(&clev,df));
                                                UPPER=exp(estimate+tot_se*tinv(&clev,df));
                  end;
                                                   %end;
         output finalb;
    run;
    ods HTML body="&out..html";

    %if &proc = corr %then %do;

                               data finala(keep = &byvar parameter &ivar %do i=1 %to &nivar;%scan(&ivar,&i)_STD %scan(&ivar,&i)_Prob %end;)
                                    finalc(keep= &byvar parameter &ivar)
                                    finald(keep= &byvar parameter &ivar)
                                    finale(keep= &byvar parameter &ivar) ;

                                    set finalb;by &byvar;
                                    if substr(parameter,1,4) ne "CORR" then do;
                                    %scan(&ivar,1)_STD= tot_se;
                                    %scan(&ivar,1)_Prob= prob_t;
                                    %do i=2 %to &nivar;%scan(&ivar,&i)_STD= tot_se&i;
                                                       %scan(&ivar,&i)_Prob=prob_t&i;
                                    %end;
                                    output finala;
                                                                            end;
                                    else do;
                                    output finalc;
                                            %scan(&ivar,1)=tot_se;
                                            spec = substr(parameter,5,28);
                                            parameter="STD "|| spec;
                                            %do i=2 %to &nivar;%scan(&ivar,&i)=tot_se&i;%end;
                                            output finald;
                                            %scan(&ivar,1)=prob_t;
                                            parameter="Prob" || spec;
                                            %do i=2 %to &nivar;%scan(&ivar,&i)=prob_t&i;%end;
                                            output finale;
                                         end;
                               data finalc;set finalc finald finale;

                               proc print data=finala;by &byvar;
                               var parameter %do i=1 %to &nivar; %scan(&ivar,&i) %scan(&ivar,&i)_STD %scan(&ivar,&i)_Prob %end;;
                               %if &npv>1 %then %do;
                               title2
                               "General Information controlling for &byvar for variables &ivar, ";
                               title3 "based on &npv sets of Plausible Values and adjusted Degrees of Freedom";
                                                %end;
                               %else %do;
                               title2
                               "General Information controlling for &byvar for variables &ivar, ";
                               title3 "based on %eval(&df) Degrees of Freedom";
                                      %end;
                               run;

                               proc sort data=finalc;by &byvar;run;
                               proc print data=finalc;by &byvar;
                               var parameter &ivar;
                               %if &npv>1 %then %do;
                               title2
                               "Correlation coefficients controlling for &byvar for variables &ivar, ";
                               title3 "based on &npv sets of Plausible Values and adjusted Degrees of Freedom";
                                                %end;
                               %else %do;
                               title2
                               "Correlation coefficients controlling for &byvar for variables &ivar, ";
                               title3 "based on %eval(&df) Degrees of Freedom";
                                      %end;
                               run;
                            %end;

    %else %if &proc = freq %then %do;
                               proc format;
                               value response .='TOTAL';
                               proc tabulate data=finalb missing order=formatted;
                               by &byvar;
                               %if &npv>1 %then %do;
                               title2
                               %if &nivar>1 %then %do;
     "Crosstabs controlling for &byvar by %scan(&ivar,1) and %scan(&ivar,2), ";
                               title3 "based on &npv sets of Plausible Values and adjusted Degrees of Freedom";
                                                  %end;
                               %else %do;
                               "Crosstabs controlling for &byvar by &ivar, based on &npv sets of ";
                               title3 " Plausible Values and adjusted Degrees of Freedom";
                                     %end;
                                                %end;
                               %else %do;
                               title2
                               %if &nivar>1 %then %do;
     "Crosstabs controlling for &byvar by %scan(&ivar,1) and %scan(&ivar,2), ";
                               title3 "based on %EVAL(&df) Degrees of Freedom";
                                                  %end;
                               %else %do;
                               "Crosstabs controlling for &byvar by &ivar, based on ";
                               title3 " %eval(&df) Degrees of Freedom";
                                     %end;
                                     %end;
                               var frequency tot_se2 percent tot_se %if &nivar>1 %then %do;
                                                                                           rowpercent tot_se3 colpercent tot_se4
                                                                                       %end;;
                               format &ivar response.;
                               label frequency='Pop Counts'
                                     percent='%'
                                     tot_se='s.e.'
                                     tot_se2='s.e.'
                                                  %if &nivar>1 %then %do;
                                     rowpercent='row %'
                                     colpercent='col %'
                                     tot_se3='s.e.'
                                     tot_se4='s.e.'
                                                                      %end;;
                               class &ivar;
                               table %scan(&ivar,1)*(frequency*mean=''*f=10. tot_se2*mean=''*f=10. percent*mean=''*f=5.1 tot_se*mean=''*f=6.2
                                                    %if &nivar>1 %then %do;
                                                     rowpercent*mean=''*f=5.1 tot_se3*mean=''*f=6.2
                                                     colpercent*mean=''*f=5.1 tot_se4*mean=''*f=6.2),
                                     %scan(&ivar,2) / box="Pop Counts & Percentages %scan(&ivar,1) * %scan(&ivar,2)";
                                                                       %end;
                                                    %else %do;
                                                    ) / box="Pop Counts & Percentages by &ivar" ;
                                                          %end;
                               run;
                         %end;
    %else %do;
    %if &proc = glm %then %do;
    proc sort data=finala;by &byvar %if &nclvar>0 %then %do; class %end; ;
    run;
    data finala;set finala;by &byvar %if &nclvar>0 %then %do; class %end; ;
    if %if &nclvar>0 %then %do; first.class; %end; %else %do; first.&byvar; %end;
    run;
    proc print data=finala ;
    %if %lowcase(&method)=regr %then %do;
    %if &npv>1 %then %do;
    title2 "Standard Regression of &dvar on &ivar Based on &npv sets of Plausible Values ";
    title3 "and adjusted D.F.. Controlling for &byvar";
                     %end;
    %else %do;
    title2 "Standard Regression of &dvar on &ivar Based on %eval(&df) degrees of Freedom.";
    title3 "Controlling for &byvar";
          %end;
         var &byvar dependent rsquare dep_mean n_obs %if &nclvar>0 %then %do; class levels values %end;;
    run;
                                     %end;
    %else %do;
    %if &npv>1 %then %do;
    title2 "Estimated mean differences of &dvar by &ivar Based on &npv sets of Plausible Values ";
    title3 "and adjusted D.F.. Controlling for &byvar - Highest levels specify the reference group";
                     %end;
    %else %do;
    title2 "Estimated mean differences of &dvar by &ivar Based on %eval(&df) degrees of Freedom.";
    title3 "Controlling for &byvar - Highest levels specify the reference group";
           %end;
         var &byvar dependent rsquare dep_mean n_obs %if &nclvar>0 %then %do; class levels values %end;;
    run;
          %end;
                         %end;
    %if &proc = catmod | &proc = logistic %then %do;
    %if &npv>1 %then %do;
    proc print data=resp1;
    %if &nevent>0 %then %do;  
    title2 " %upcase(&method) Regression (Event=&event) Based on &npv sets of Plausible Values, Description of ";
                        %end;
    %else %do;
          title2 " %upcase(&method) Regression Based on &npv sets of Plausible Values, Description of ";
          %end;
    title3 "the dependent variable &dvar . Controlling for &byvar . Adjusted Degrees of Freedom";
    run;
    /*
    %if &proc = catmod %then %do;
    proc print data=popu1;
    title2 "Logistic Regression Based on &npv sets of Plausible Values, Description of ";
    title3 "the independent variable(s) &ivar . Controlling for &byvar . Adjusted Degrees of Freedom";
         var &byvar &ivar ;
    run;
                            %end;
    */
    %if &proc = logistic %then %do;
    %if &nivar >0 %then %do;
        proc print data=all_info label;
        title2 "Log Likelihood - Measure of Fit";
            var &byvar value interceptonly interceptandcovariates negative;
            format interceptonly 12. interceptandcovariates 12. negative 4.2;
                         %end;
    %if &nclvar>0 %then %do;
    data finala;set finala;by &byvar;retain ref;if class ne ' ' then ref=class;class=ref;
                drop ref estimate id parameter smp_var impu_var tot_var tot_se prob_t odds_ratio upper lower fm df ddf;run;
    proc sort data=finala;by &byvar class value;
    data finala;set finala;by &byvar class value;if first.value;
    proc print data=finala;
    title2 " %upcase(&method) Regression Based on &npv sets of Plausible Values, Description of ";
    title3 "the independent variable(s) &clvar . Controlling for &byvar . Adjusted Degrees of Freedom";
    run;
                        %end;
                            %end;
                     %end;
    %else %do;
    proc print data=resp1;
    %if %nevent>0 %then %do;
    title2 " %upcase(&method) Regression (Event=&event) Based on %eval(&df) Degrees of Freedom, Description of ";
                        %end;
    %else %do;
          title2 " %upcase(&method) Regression Based on %eval(&df) Degrees of Freedom, Description of ";
          %end;
    title3 "the dependent variable &dvar . Controlling for &byvar";
    run;
    /*
    %if &proc = catmod %then %do;
    proc print data=popu1;
    title2 "Logistic Regression Based on %eval(&df) Degrees of freedom, Description of ";
    title3 "the independent variable(s) &ivar . Controlling for &byvar";
         var &byvar &ivar ;
    run;
                             %end;
    */

    %if &proc = logistic %then %do;
    %if &nivar >0 %then %do;
        proc print data=all_info label; 
        title2 "Log Likelihood - Measure of Fit";
            var &byvar interceptonly interceptandcovariates negative;
            format interceptonly 12. interceptandcovariates 12. negative 4.2;
                        %end;
    %if &nclvar>0 %then %do;
    data finala;set finala;by &byvar;retain ref;if class ne ' ' then ref=class;class=ref;
                drop ref estimate id parameter smp_var tot_var tot_se prob_t odds_ratio upper lower;run;
    proc sort data=finala;by &byvar class value;
    data finala;set finala;by &byvar class value;if first.value;
    proc print data=finala;
    title2 " %upcase (&method) Regression Based on %eval(&df) Degrees of freedom, Description of ";
    title3 "the independent variable(s) &clvar . Controlling for &byvar";
    run;
                       %end;
                             %end;
          %end;
                                              %end;
    %if &proc = logistic %then %do;
    %if &prefix=1 %then %do;
    data finalb;set finalb;
         lev=substr(parameter,1,1);
    proc sort data=finalb;by &byvar lev;
    run;
                        %end;
                              %end;
/***********************************************************************************************************************************/
    %if &international = 1 %then %do;
    proc format;
         value int .="Int'l Avg.";
                                 %end;

    proc print data=finalb label;
    label
         Tot_se = "Standard Error"
         prob_T = "Prob > |T|"
         df = "D.F."
         %if &proc = catmod | &proc = logistic %then %do;
         odds_ratio ="Odds Ratio"
         LOWER = "95% Wald Lower Limit"
         UPPER = "95% Wald Upper Limit"
         %if &prefix=1 %then %do;lev="&dvar" %end;
                                                  %end;
         ;
    %if &npv>1 %then %do;
    title2
          %if %lowcase(&method) = regr %then %do;
                "Estimated Regression Coefficients Based on &npv sets of Plausible Values and adjusted D.F.";
                 title3 "MODEL : &dvar = X'B = (Parameter)' (Estimate)";
                                             %end;
          %else %if %lowcase(&method) = diff %then %do;
                "Estimated Mean Differences Based on &npv sets of Plausible Values and adjusted D.F.";
                 title3 "Last line gives the reference group";
                                                  %end;
          %else %if &proc = catmod | &proc = logistic %then %do;
                  %if &nwhere = 2 | &nwhere = 0 %then %do;
                 "Estimated %upcase(&method) Regression Coefficients Based on &npv sets of Plausible Values and adjusted D.F.";
                  title3 "MODEL : Ln(P/1-P) = X'B = (Parameter)' (Estimate) with P based on &dvar levels (2 levels) ";
                                                      %end;
                  %else %do;
                       %if &proc = logistic %then %do;
                 "Estimated Cumulative Logit Regression Coefficients Based on &npv sets of Plausible Values and adjusted D.F.";
          title3 "MODEL : Ln(Fi/1-Fi) = X'B = (Parameter)' (Estimate) with Fi based on &dvar levels (> 2 levels)";
                                                  %end;
                       %else %do;
                  "Estimated Multinomial Logistic Regression Coefficients Based on &npv sets of Plausible Values and adjusted D.F.";
          title3 "MODEL : Ln(Pi/Pk) = X'B = (Parameter)' (Estimate) with Pi based on &dvar levels (> 2 levels)";
                             %end;                  
                        %end;
                                                          %end;
          %else %if &proc = means %then %do;
                    %if %lowcase(&stat)=mean %then %do;
                 " Estimated Means for &dvar by &byvar and &ivar, Based on &npv sets of Plausible Values and adjusted D.F.";
                 title3 ;
                                                   %end;
                    %else %if %lowcase(%substr(&stat,1,1))=p %then %do;
                 " Estimated Percentiles (&stat) for &dvar by &byvar and &ivar, Based on &npv sets of Plausible Values and adjusted D.F.";
                 title3 ;
                                                                   %end;

                    %else %if %lowcase(&stat)=std %then %do;
                 " Estimated Standard Deviations for &dvar by &byvar and &ivar, Based on &npv sets of Plausible Values and adjusted D.F.";
                 title3 ;
                                                                   %end;


                    %else %do;
                 " Estimated Population Counts by &byvar and &ivar, Based on &npv sets of Plausible Values
 and adjusted D.F.";
                 title3 ;
                          %end;
                                        %end;
    title4 "Controlling for &byvar.";
                    %end;
    %else %do;
    title2
          %if %lowcase(&method) = regr %then %do;
                "Estimated Regression Coefficients Based on %eval(&df) Degrees of Freedom.";
                 title3 "MODEL : &dvar = X'B = (Parameter)' (Estimate)";
                                             %end;
          %else %if %lowcase(&method) = diff %then %do;
                "Estimated Mean Differences Based on &npv sets of Plausible Values and %eval(&df) D.F.";
                 title3 "Last line gives the reference group";
                                                   %end;
          %else %if &proc = catmod | &proc = logistic %then %do;
                  %if &nwhere = 2  | &nwhere = 0 %then %do;
                 "Estimated %upcase(&method) Regression Coefficients Based on %eval(&df) Degrees of Freedom.";
                  title3 "MODEL : Ln(P/1-P) = X'B = (Parameter)' (Estimate) with P based on &dvar levels (2 levels)";
                                                       %end;
                  %else %do;
                       %if &proc = logistic %then %do;
                 "Estimated Cumulative Logit Regression Coefficients Based on %eval(&df) Degrees of Freedom.";
          title3 "MODEL : Ln(Fi/1-Fi) = X'B = (Parameter)' (Estimate) with Fi based on &dvar levels (> 2 levels)";
                                                %end;
                       %else %do;
                  "Estimated Multinomial Logistic Regression Coefficients Based on %eval(&df) Degrees of Freedom.";
           title3 "MODEL : Ln(Pi/Pk) = X'B = (Parameter)' (Estimate) with Pi based on &dvar levels (> 2 levels)";
                             %end;                  
                        %end;
                                                          %end;
          %else %if &proc = means %then %do;
                    %if %lowcase(&stat)=mean %then %do;
                    %if &pct_correct=1 %then %do;
                 " Estimated Average Percent Correct by &byvar and &ivar, Based on %eval(&df) Degrees of Freedom";
                 title3 ;
                                             %end;
                    %else %do;
                 " Estimated Means for &dvar by &byvar and &ivar, Based on %eval(&df) Degrees of Freedom.";
                 title3 ;
                          %end;
                                                   %end;
                    %else %if %lowcase(%substr(&stat,1,1))=p %then %do;
                 " Estimated Percentiles (&stat) for &dvar by &byvar and &ivar, Based on %eval(&df) Degrees of Freedom.";
                 title3 ;
                                                   %end;
                    %else %if %lowcase(&stat)=std %then %do;
                 " Estimated Standard Deviations for &dvar by &byvar and &ivar, Based on &npv sets of Plausible Values and adjusted D.F.";
                 title3 ;
                                                                   %end;

                    %else %do;
                 " Estimated Population Counts by &byvar and &ivar, Based on %eval(&df) Degrees of Freedom.";
                 title3 ;
                          %end;
                                        %end;
    title4 "Controlling for &byvar";
         %end;
         var %if %scan(&byvar,1) = %scan(&ivar,1) && %lowcase(%scan(&byvar,1)) = domain %then %do;%end; %else %do; &byvar %end; 
                    %if &proc = logistic &&  &prefix=1 %then %do;lev %end;
                    %if &proc = glm | &proc = catmod | &proc = logistic %then %do; parameter %end;
                    %else %do; &ivar %end;
                               estimate tot_se 
                               %if &proc = catmod | &proc = logistic %then %do; odds_ratio Lower Upper %end;
                               prob_T df
                                      ;
         format estimate 12.2 tot_se 12.3 prob_t 7.5 df 6.2 %if &proc = catmod | &proc = logistic %then %do; 
                                                            odds_ratio 7.2 Lower 7.2 Upper 7.2 %end;;
         %if &international = 1 %then %do;format %scan(&byvar,1) int.;%end;
    run;
              %end;
    ods html close;


%if %lowcase(&method) = corrdiff %then %do;

    %let max = %eval(((&nivar*(&nivar-1))/2)-1);

    data all_rep;set all_rep;by &byvar;
     retain %do i=1 %to &nivar-1; %do j=&i+1 %to &nivar; rho&i.0&j %end;%end;;
     keep &byvar %do i=1 %to &nivar-1; %do j=&i+1 %to &nivar; rho&i.0&j %end;%end;;
     array rho(*) rho1-rho9999;
     if substr(parameter,1,4) ne "CORR" then do;k=0;delete;end;
     else if k=&nivar then delete;
     else do;
             k+1;
             %do j=1 %to &nivar;
                z=k*100+&j;
                rho(z)=%scan(&ivar,&j);
             %end;
          if k=&nivar-1 then output;
          end;
  run;

 data all_rep;set all_rep;by &byvar;
      drop rho_%scan(&ivar,1)_%scan(&ivar,2) %do i=1 %to &nivar-1; %do j=&i+1 %to &nivar; rho&i.0&j %end;%end;;
      length parameter $ 32;
      %do i=1 %to &nivar-2;
        %do j=&i+1 %to &nivar;
              %let z=%eval(&i*100+&j);
              parameter = "DIFF rho_%scan(&ivar,&i)_%scan(&ivar,&j)";

              %do ii=&i %to &nivar-1;
              %do jj=&ii+1 %to &nivar;
                  %let zz=%eval(&ii*100+&jj);
                  %if &zz<=&z %then %do;
                                        rho_%scan(&ivar,&ii)_%scan(&ivar,&jj) = .;
                                    %end;
                                    %else %do;
                                            rho_%scan(&ivar,&ii)_%scan(&ivar,&jj) = rho&zz - rho&z;
                                          %end;
              %end;
              %end;
          output;
          %end;
        %end;

run;


   proc summary data=all_rep nway;
        by &byvar;
        class parameter;
        var %do i=1 %to &nivar-1;
               %do j=&i+1 %to &nivar;
                   %if &i=1 & &j=2 %then %do;%end;
                                         %else %do;
                                                   rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                               %end;
               %end;
             %end;;
        output out=tot_rep css = %do i=1 %to &max; css&i %end;
                           mean= %do i=1 %to &max; ybar&i %end;
                           ;
   run;

*    Computes the overall mean based on &wgt ;
   %do lp=1 %to &npv;
     proc sort data=impu&lp;by &byvar id;run;

     data impu&lp;set impu&lp;by &byvar;
     retain %do i=1 %to &nivar-1; %do j=&i+1 %to &nivar; rho&i.0&j %end;%end;;
     keep &byvar id %do i=1 %to &nivar-1; %do j=&i+1 %to &nivar; rho&i.0&j %end;%end;;
     array rho(*) rho1-rho9999;
     if substr(parameter,1,4) ne "CORR" then do;k=0;delete;end;
     else if k=&nivar then delete;
     else do;
             k+1;
             %do j=1 %to &nivar;
                z=k*100+&j;
                rho(z)=%scan(&ivar,&j);
             %end;
          if k=&nivar-1 then output;
          end;
  run;

 data impu&lp;set impu&lp;by &byvar;
      drop rho_%scan(&ivar,1)_%scan(&ivar,2) %do i=1 %to &nivar-1; %do j=&i+1 %to &nivar; rho&i.0&j %end;%end;;
      length parameter $ 32;
      %do i=1 %to &nivar-2;
        %do j=&i+1 %to &nivar;
              %let z=%eval(&i*100+&j);
              parameter = "DIFF rho_%scan(&ivar,&i)_%scan(&ivar,&j)";

              %do ii=&i %to &nivar-1;
              %do jj=&ii+1 %to &nivar;
                  %let zz=%eval(&ii*100+&jj);
                  %if &zz<=&z %then %do;
                                        rho_%scan(&ivar,&ii)_%scan(&ivar,&jj) = .;
                                    %end;
                                    %else %do;
                                            rho_%scan(&ivar,&ii)_%scan(&ivar,&jj) = rho&zz - rho&z;
                                          %end;
              %end;
              %end;
          output;
          %end;
        %end;

   %end;
   proc sort data=impu1;by &byvar parameter;
   run;
*    Computes the sampling variances;
   data smp_var;merge impu1 tot_rep;by &byvar  parameter;
        %do i=1 %to &max;
            %if &i=1 %then %do;%let ii=1;%let jj=3;%end;
            %else %do;
                      %if &nivar<=&jj %then %do;%let ii=%eval(&ii+1);%let jj=%eval(&ii+1);%end;
                      %else %do;%let jj = %eval(&jj+1);%end;
                  %end;
        smp_var&i=&c*(css&i + &nrep*(ybar&i-rho_%scan(&ivar,&ii)_%scan(&ivar,&jj))*(ybar&i-rho_%scan(&ivar,&ii)_%scan(&ivar,&jj)));
        %end;

        keep &byvar id parameter %do i=1 %to &nivar-1;
                                     %do j=&i+1 %to &nivar;
                                         %if &i=1 & &j=2 %then %do;%end;
                                                               %else %do;
                                                                         rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                                     %end;
                                     %end;
                                 %end;
                                 %do i=1 %to &max; smp_var&i  %end;;
   run;

* Computes the imputation variance based on all plausible values;

   %if &npv>1 %then %do;
       data all_impu;
            set
               %do i=1 %to &npv;
                   impu&i
               %end;
               ;
       run;
       
       proc sort data=all_impu;by &byvar parameter; run;
       proc summary data=all_impu nway;
            by &byvar;
            class parameter;
            var %do i=1 %to &nivar-1;
                    %do j=&i+1 %to &nivar;
                        %if &i=1 & &j=2 %then %do;%end;
                                              %else %do;
                                                        rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                    %end;
                    %end;
                %end;;

            output out=tot_impu css = %do i=1 %to &max; css&i %end;
                                mean= %do i=1 %to &max; %do i=1 %to &nivar-1;
                                                           %do j=&i+1 %to &nivar;
                                                               %if &i=1 & &j=2 %then %do;%end;
                                                                                     %else %do;
                                                                                               rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                                                           %end;
                                                           %end;
                                                        %end;;

                                      %end;

   run;

   data impu_var;set tot_impu;
            %do i=1 %to &max;
            impu_var&i = (&npv+1)*css&i/(&npv*(&npv-1));
            %end;

            keep &byvar %do i=1 %to &nivar-1;
                            %do j=&i+1 %to &nivar;
                                %if &i=1 & &j=2 %then %do;%end;
                                                      %else %do;
                                                                rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                            %end;
                            %end;
                        %end;
                 parameter
                 %do i=1 %to &max;impu_var&i %end;;
  run;
* Computes the overall variance ;
       data tot_var;merge smp_var impu_var;by &byvar parameter;
                                       %do i=1 %to &max;
                                       tot_var&i = smp_var&i + impu_var&i;
                                       if tot_var&i =0 then fm=0;else
                                       fm = impu_var&i / tot_var&i;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df&i = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df&i = &ddf; %end; 
                                       tot_se&i  = sqrt(tot_var&i);
                                       if tot_se&i ne 0 then do;
                                                       %if &i=1 %then %do;%let ii=1;%let jj=3;%end;
                                                       %else %do;
                                                                 %if &nivar<=&jj %then %do;%let ii=%eval(&ii+1);%let jj=%eval(&ii+1);%end;
                                                                 %else %do;%let jj = %eval(&jj+1);%end;
                                                              %end;
                                                       prob_T&i  = 2*(1-probt(abs(rho_%scan(&ivar,&ii)_%scan(&ivar,&jj)/tot_se&i),df&i));
                                                             end;
                                       else prob_T&i=.;
                                       %end;
       run;
                     %end;
    %else %do;
       data tot_var;set smp_var;
                                       %do i=1 %to &max;
                                       tot_var&i = smp_var&i ;
                                       df&i = &df;
                                       tot_se&i  = sqrt(tot_var&i);
                                       if tot_se&i ne 0 then do;
                                                       %if &i=1 %then %do;%let ii=1;%let jj=3;%end;
                                                       %else %do;
                                                                 %if &nivar<=&jj %then %do;%let ii=%eval(&ii+1);%let jj=%eval(&ii+1);%end;
                                                                 %else %do;%let jj = %eval(&jj+1);%end;
                                                              %end;
                                                       prob_T&i  = 2*(1-probt(abs(rho_%scan(&ivar,&ii)_%scan(&ivar,&jj)/tot_se&i),&df));
                                                             end;
                                       else prob_T&i=.;

                                       %end;



       run;
          %end;
    proc sort data=tot_var;by &byvar id; run;

    data finalb(keep = &byvar parameter
         %do i=1 %to &nivar-1;
             %do j=&i+1 %to &nivar;
                 %if &i=1 & &j=2 %then %do;%end;
                                       %else %do;
                                                 rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                             %end;
             %end;
         %end; %do i=1 %to &max;tot_se&i df&i prob_T&i %end;);

        set tot_var ;
        by &byvar;
        output finalb;
    run;

ods HTML body="&out.diff.html";

    data
         finalc(keep= &byvar parameter %do i=1 %to &nivar-1;
                                           %do j=&i+1 %to &nivar;
                                               %if &i=1 & &j=2 %then %do;%end;
                                                                     %else %do;
                                                                               rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                                           %end;
                                           %end;
                                        %end; )
         finald(keep= &byvar parameter %do i=1 %to &nivar-1;
                                           %do j=&i+1 %to &nivar;
                                               %if &i=1 & &j=2 %then %do;%end;
                                                                     %else %do;
                                                                               rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                                           %end;
                                           %end;
                                        %end; )
         finale(keep= &byvar parameter %do i=1 %to &nivar-1;
                                           %do j=&i+1 %to &nivar;
                                               %if &i=1 & &j=2 %then %do;%end;
                                                                     %else %do;
                                                                               rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                                           %end;
                                           %end;
                                        %end; ) ;

      set finalb;by &byvar;
      output finalc;
      spec = substr(parameter,5,28);
      parameter="STD "|| spec;
      %do i=1 %to &max;
          %if &i=1 %then %do;%let ii=1;%let jj=3;%end;
          %else %do;
                    %if &nivar<=&jj %then %do;%let ii=%eval(&ii+1);%let jj=%eval(&ii+1);%end;
                    %else %do;%let jj = %eval(&jj+1);%end;
                %end;
          rho_%scan(&ivar,&ii)_%scan(&ivar,&jj) = tot_se&i;
      %end;
      output finald;
      parameter="Prob" || spec;
      %do i=1 %to &max;
          %if &i=1 %then %do;%let ii=1;%let jj=3;%end;
          %else %do;
                    %if &nivar<=&jj %then %do;%let ii=%eval(&ii+1);%let jj=%eval(&ii+1);%end;
                    %else %do;%let jj = %eval(&jj+1);%end;
                %end;
          rho_%scan(&ivar,&ii)_%scan(&ivar,&jj) = prob_t&i;
      %end;
      output finale;
      run;
      data finalc;set finalc finald finale;

      proc sort data=finalc;by &byvar;run;
      proc print data=finalc;by &byvar;
           var parameter %do i=1 %to &nivar-1;
                                 %do j=&i+1 %to &nivar;
                                     %if &i=1 & &j=2 %then %do;%end;
                                                            %else %do;
                                                                      rho_%scan(&ivar,&i)_%scan(&ivar,&j)
                                                                  %end;
                                  %end;
                         %end;;
               %if &npv>1 %then %do;
                               title2
                               "Difference between correlation coefficients controlling for &byvar for variables &ivar, ";
                               title3 "based on &npv sets of Plausible Values and adjusted Degrees of Freedom";
                                %end;
                               %else %do;
                               title2
                               "Difference between correlation coefficients controlling for &byvar for variables &ivar, ";
                               title3 "based on %eval(&df) Degrees of Freedom";
                                      %end;
                               run;
   ods html close;

                            %end;
      %end;

%if &proc = univariate %then %do;
 %let lp=1;
 %if &byflag>0 | &clflag>0 | &iflag>0  %then %do;
              data all91z;set all91z;
                 %if &byflag>0 %then %do;
                 %let i = 1;
                      %do %while(%length(%scan(&byvar,&i)));
                          %if %scan(&byvarpv, &i) =1 %then %do; %scan(&byvar, &i)=%scan(&byvar,&i)&lp;
                                                           %end;
                          %let i = %eval(&i + 1);
                      %end;
                ;
                                     %end;
                 %if &iflag>0 %then %do;

                 %let i = 1;
                      %do %while(%length(%scan(&ivar,&i)));
                          %if %scan(&ivarpv, &i) =1 %then %do; %scan(&ivar, &i)=%scan(&ivar,&i)&lp;
                                                           %end;
                          %let i = %eval(&i + 1);
                      %end;
                ;
                                    %end;
                 

                 %let i = 1;
                      %do %while(%length(%scan(&clvar,&i)));
                          %if %scan(&ref,1) = default %then %do; %end;
                          %else %do;
                          if %scan(&clvar,&i) = %scan(&ref,&i) then %scan(&clvar,&i) = 9999;
                                %end;
                          %let i = %eval(&i + 1);
                      %end;
                ;
                                    
                                            %end;
* Computes the imputation variance based on all plausible values;
   %if &npv>1 %then %do;
       data all_impu;
            set
               %do i=1 %to &npv;
                   impu&i
               %end;
               ;
       run;
       proc sort data=all_impu;by &byvar &ivar;
       %if &proc = freq && &npv > 1 %then %do;
       data all_impu (drop=comptr) ;set all_impu;
            by &byvar &ivar;
            output;
            comptr +1;
            if last.&lastivar then do;
               do while (comptr < &npv);percent=0; frequency=0; %if &nivar > 1 %then %do; rowpercent=.; colpercent=.; %end;
                                        output;comptr+1;
               end;
               comptr=0;
                                   end;
       run;
                                      %end;
       
       proc summary data=all_impu nway;
            by &byvar &ivar;
            var P_&stat;
        output out=tot_impu css=css
                           mean= estimate;
   run;
       data impu_var;set tot_impu;
            impu_var = (&npv+1)*css/(&npv*(&npv-1));
            keep &byvar &ivar
                        impu_var
                        estimate;
       run;
                     %end;
* Read the data file and keep only what is necessary sorting by the
  subpopulation variables (ByVAR);
proc sort data = all91z
     out = a;
     by &byvar;
run;
* Finds out which analytical tool was requested and runs it;
   %do lp=1 %to &nrep;
        ods listing close;
        proc means data=a ;
        by &byvar;
        %if &nclvar = 0 %then %do;%end;%else %do; class &clvar; %end;
        var pctl1;
        weight
              %if &nrep <    10 %then %let ai = %eval(&lp); %else
              %if &nrep <   100 %then %do;
                  %if %eval(&lp) < 10 %then %let ai=0&lp; %else
                  %let ai = %eval(&lp);
                                      %end; %else
              %if &nrep <  1000 %then %do;
                  %if %eval(&lp) <  10 %then %let ai=00&lp; %else
                  %if %eval(&lp) < 100 %then %let ai=0&lp; %else
                  %let ai = %eval(&lp);
                                      %end; %else
              %if &nrep < 10000 %then %do;
                  %if %eval(&lp) <   10 %then %let ai=000&lp; %else
                  %if %eval(&lp) <  100 %then %let ai=00&lp; %else
                  %if %eval(&lp) < 1000 %then %let ai=0&lp; %else
                  %let ai = %eval(&lp);
                                      %end;
              &rwgt&ai;
        %if &nivar=1 %then %do; ways 1; %end; %else %do; ways &nclvar; %end;
        output out=stat&lp mean=estimate;
        quit;
        ods output close;
        ods listing;
   %end;
        ods listing close;
        proc means data=a ;
        by &byvar;
        %if &nclvar=0 %then %do;%end;%else %do; class &clvar; %end;
        var pctl1;
        weight &wgt;
        %if &nivar=1 %then %do; ways 1; %end; %else %do; ways &nclvar; %end;
        output out=impu1 mean=estimate;
        quit;
        %if &nclvar=0 %then %do;%end;%else %do;ods output close; %end;
        ods listing;
* Computes sampling variance based on first plausible values;
*    Computes the jackknife repliclates;
   data all_rep;
        set
           %do i=1 %to &nrep;
               stat&i
           %end;
           ;
   run;
   proc sort data=all_rep;by &byvar &ivar;run;
   proc summary data=all_rep nway;
        by &byvar &ivar;
        var estimate;
        output out=tot_rep css=css
                           mean=ybar ;
   run;
*    Computes the overall mean based on &wgt ;
   %do lp=1 %to &npv;
       data impu&lp;set impu&lp;if _n_=1 then id+1+&lp*100000;else id+1;
       run;
   %end;
   proc sort data=impu1;by &byvar &ivar;
   run;
*    Computes the sampling variances;
   data smp_var;merge impu1 tot_rep;by &byvar &ivar;
        smp_var=&c*(css + &nrep*(ybar-estimate)*(ybar-estimate));
        keep &byvar id &ivar
              smp_var estimate ;
   run;
* Computes the overall variance ;
       data tot_var;set smp_var;
            tot_var = smp_var;
            df = &df;
            tot_se  = sqrt(tot_var);
            if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),&df));
            else prob_T=.;
       run;
   data points;set tot_var;
        Plo = 100*(estimate - TINV(&clev,&df)*tot_se);
        Phi = 100*(estimate + TINV(&clev,&df)*tot_se);
        if Plo < 0.0 then Plo=0.0001;
        if Phi > 100 then Phi=99.9999;
        keep plo phi &byvar &ivar ;
   run;
   ods listing close;
   proc means data=setup;
        class &byvar &ivar;
        ways %eval(&nbyvar+ &nivar);
        var &dvar;
        weight &wgt;
        output out=sum_in sumwgt=tot;
   run;
   ods listing;
   proc sort data=setup;
        by &byvar &ivar &dvar;
   data finald;merge setup points sum_in;
        by &byvar &ivar;
        retain xlo xhi x&stat flaglo flag&stat flaghi 0;
        cumul+&wgt;
        if 100*cumul/tot > plo & flaglo=0 then do;xlo=&dvar;flaglo=1;end;
        if 100*cumul/tot > &stat & flag&stat=0 then do;x&stat=&dvar;flag&stat=1;end;
        if 100*cumul/tot > phi & flaghi=0 then do;xhi=&dvar;flaghi=1;end;
        if last.&lastivar then do;std_err = (xhi-xlo)/(2*tinv(&clev,&df));output finald;
                              cumul=0;flaglo=0;flag&stat=0;flaghi=0;
                              xlo=0;x&stat=0;xhi=0 ;
                           end;
   %if &dflag > 0 | &byflag >0 | &iflag >0 | &clflag > 0 %then %do;
   data finald;merge finald impu_var;
        by &byvar;
        x&stat = estimate;
        std_err=sqrt(std_err**2 + impu_var);
   run;
                                                               %end;
   ods HTML body="&out..html";
   proc print data=finald label ;
   var %if %scan(&byvar,1) = %scan(&ivar,1) && %lowcase(%scan(&byvar,1)) = domain %then %do;%end; %else %do; &byvar %end; 
             &ivar x&stat std_err %if &dflag = 0 %then %do; xlo xhi %end; ;
   format X&STAT 12.2  STD_ERR 12.3  %if &dflag = 0 %then %do; xlo 12.2 xhi 12.2 %end;;
   label
         x&stat = "&dvar &stat th Percentile"
         std_err = "Standard Error"
         xlo = " %sysevalf(100*(2*&clev-1))% lower confidence limit"
         xhi = " %sysevalf(100*(2*&clev-1))% higher confidence limit"
   ;
   %if &npv>1 %then %do;
   title2 " Estimated &stat th Percentiles for &dvar controlling for &byvar by &ivar, based on ";
   title3 " &npv sets of Plausible Values and %eval(&df) D.F.. Method based on Woodruff approach.";
   title4;
                    %end;
   %else %do;
   title2 " Estimated &stat th Percentiles for &dvar controlling for &byvar by &ivar, based on ";
   title3 " %eval(&df) Degrees of Freedom. Method based on Woodruff approach.";
   title4;
         %end;
run;
   ods html close;
                             %end;
%end;

/************************************************************************************************************************************/

%else %do;
%if (%length(%scan(&byvar,1)))=0 %then %do;%let byvar=domain; %end;
%IF &PROC =univariate %then %do;data _null_;file print;put "No Statistical Test with this Procedure";
PUT "To get statistical tests, please first get the percentile estimates using method=mean";
put "with the stat=Pxx where xx is between 1 and 99. For example stat=p25 gives the 25th percentile";
put "Note that under this method, the standard error are less stable";run; %end;
%else %do;
 
%if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; proc sort data=all_rep;by &byvar parameter;run; %end;
    
data all_rept;
         set all_rep;
         by &byvar
            %if &proc = means | &proc = freq | &proc=univariate %then %do; &ivar; %end;
            %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; parameter; %end;
         length label $48.;
         %if &proc = means | &proc = freq | &proc=univariate  %then %do; if first.&lastivar then seq=0; %end;
         %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; if first.parameter then seq=0; %end;
         seq+1;
         %let j=1;
         
         %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; parameter=compress(parameter); %end;
         %do %while (&j > 0);
             if &&compare&j then do;label="&j- &&label&j";order=1;output;end;
             if &&to&j      then do;label="&j- &&label&j";order=2;output;end;
             %let j = %eval(&j + 1);
             %if %lowcase(&&compare&j) = no  %then %let j=0;
         %end;

    proc sort data=all_rept;by &byvar label seq order;

    data all_rept;set all_rept; by &byvar label seq;
         retain ref1-ref4 0;
         %if &proc = freq %then %do; if first.seq then do;ref1=frequency;
                                                      ref2=percent;
                                                      %if &nivar > 1 %then %do;
                                                           ref3=rowpercent;
                                                           ref4=colpercent;
                                                                           %end;
                                                       end;
                                     else do;
                                                      frequency=ref1-frequency;
                                                      percent = ref2-percent;
                                                      %if &nivar > 1 %then %do;
                                                           rowpercent=ref3-rowpercent;
                                                           colpercent=ref4-colpercent;
                                                                           %end;
                                                       output;
                                          end;
                                %end;
         %else %do;if first.seq then ref1=estimate;else do;estimate=ref1-estimate;output; end; %end;

    %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; proc sort data=all_impu;by &byvar parameter;run; %end;


    %if &npv > 1 %then %do;
    data all_imput;
         set all_impu;
         by &byvar
            %if &proc = means | &proc = freq | &proc=univariate %then %do; &ivar; %end;
            %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; parameter; %end;
         length label $48.;
         %if &proc = means | &proc = freq | &proc=univariate  %then %do; if first.&lastivar then seq=0; %end;
         %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; if first.parameter then seq=0; %end;
         %if &method = perc %then %do; estimate = P_&stat; %end;
         seq+1;
         %let j=1;
         
         %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; parameter=compress(parameter); %end;
         %do %while (&j > 0);
             if &&compare&j then do;label="&j- &&label&j";order=1;output;end;
             if &&to&j      then do;label="&j- &&label&j";order=2;output;end;
             %let j = %eval(&j + 1);
             %if %lowcase(&&compare&j) = no  %then %let j=0;
         %end;

    proc sort data=all_imput;by &byvar label seq order;

    data all_imput;set all_imput; by &byvar label seq;
         retain ref1-ref4 0;
         %if &proc = freq %then %do; if first.seq=1 then do;ref1=frequency;
                                                      ref2=percent;
                                                      %if &nivar > 1 %then %do;
                                                           ref3=rowpercent;
                                                           ref4=colpercent;
                                                                           %end;
                                                   end;
                                     else do;
                                                      frequency=ref1-frequency;
                                                      percent = ref2-percent;
                                                      %if &nivar > 1 %then %do;
                                                           rowpercent=ref3-rowpercent;
                                                           colpercent=ref4-colpercent;
                                                                           %end;
                                                       output;
                                          end;
                                %end;
         %else %do;if first.seq then ref1=estimate;else do;estimate=ref1-estimate;output; end;%end;

   %end;

    data impu1t;
         set impu1;
         by &byvar
            %if &proc = means | &proc = freq | &proc=univariate %then %do; &ivar; %end;
            %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; parameter; %end;
         length label $48.;
         %if &proc = means | &proc = freq | &proc=univariate  %then %do; if first.&lastivar then seq=0; %end;
         %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; if first.parameter then seq=0; %end;
         seq+1;
         %let j=1;
         
         %if &proc = glm   | &proc = catmod | &proc = logistic  %then %do; parameter=compress(parameter); %end;
         %do %while (&j > 0);
             if &&compare&j then do;label="&j- &&label&j";order=1;output;end;
             if &&to&j      then do;label="&j- &&label&j";order=2;output;end;
             %let j = %eval(&j + 1);
             %if %lowcase(&&compare&j) = no  %then %let j=0;
         %end;

    proc sort data=impu1t;by &byvar label seq order;

    data impu1t;set impu1t; by &byvar label seq;
         retain ref1-ref4 0;
         %if &proc = freq %then %do; if first.seq=1 then do;ref1=frequency;
                                                      ref2=percent;
                                                      %if &nivar > 1 %then %do;
                                                           ref3=rowpercent;
                                                           ref4=colpercent;
                                                                           %end;
                                                   end;
                                     else do;
                                                      frequency=ref1-frequency;
                                                      percent = ref2-percent;
                                                      %if &nivar > 1 %then %do;
                                                           rowpercent=ref3-rowpercent;
                                                           colpercent=ref4-colpercent;
                                                                           %end;
                                                       output;
                                          end;
                                %end;
         %else %do;if first.seq then ref1=estimate;else do;estimate=ref1-estimate;output;end; %end;
       
proc summary data=all_rept nway;
        by &byvar label;
        %if &proc = glm | &proc = catmod | &proc = logistic %then %do; class parameter;
                                                                     var estimate;
                                                                %end;

        %else %if &proc = means | &proc=univariate %then %do; var estimate; %end;
        %else %if &proc = freq %then %do; var percent frequency %if &nivar > 1 %then %do; rowpercent colpercent %end;;
                                     %end;

        output out=tot_rep css=css %if &proc = freq %then %do; css2 %if &nivar>1 %then %do;css3 css4 %end;%end;
                           mean=ybar %if &proc = freq %then %do; ybar2 %if &nivar>1 %then %do;ybar3 ybar4 %end;%end;;
run;

*    Computes the overall mean based on &wgt ;

*    Computes the sampling variances;
   data smp_var;merge impu1t tot_rep;by &byvar label;

        %if &proc = freq %then %do;
        smp_var=&c*(css + &nrep*(ybar-percent)*(ybar-percent));
        smp_var2=&c*(css2 + &nrep*(ybar2-frequency)*(ybar2-frequency));
                                    %if &nivar > 1 %then %do;
        smp_var3=&c*(css3 + &nrep*(ybar3-rowpercent)*(ybar3-rowpercent));
        smp_var4=&c*(css4 + &nrep*(ybar4-colpercent)*(ybar4-colpercent));
                                                         %end;

                               %end;

        %else %do;
        smp_var=&c*(css + &nrep*(ybar-estimate)*(ybar-estimate));
              %end;
        keep &byvar label
              smp_var %if &proc = freq %then %do; percent frequency smp_var2 %if &nivar > 1 %then %do;
                                                                           smp_var3 smp_var4 rowpercent colpercent
                                                                                                  %end;;
                                             %end;

             %else %do; estimate ; %end;
   run;
* Computes the imputation variance based on all plausible values;

   %if &npv>1 %then %do;

       proc summary data=all_imput nway;
            by &byvar label;
        %if &proc = glm  | &proc = catmod | &proc = logistic  %then %do; class parameter;
                                                                       var estimate;
                                                                  %end;

        %else %if &proc = means | &proc=univariate %then %do; var estimate; %end;
        %else %if &proc = freq %then %do; var percent frequency %if &nivar > 1 %then %do; rowpercent colpercent %end;;
                                     %end;

        output out=tot_impu css=css %if &proc = freq %then %do; css2 %if &nivar>1 %then %do;css3 css4 %end;%end;
                           mean= %if &proc = freq %then %do; percent frequency %end; %else %do; estimate %end;
                                 %if &proc = freq && &nivar > 1 %then %do; rowpercent colpercent %end;
                                 ;

   run;

       data impu_var;set tot_impu;
            impu_var = (&npv+1)*css/(&npv*(&npv-1));
            %if &proc = freq %then %do;
            impu_var2 = (&npv+1)*css2/(&npv*(&npv-1));
            %if &nivar>1 %then %do;
            impu_var3 = (&npv+1)*css3/(&npv*(&npv-1));
            impu_var4 = (&npv+1)*css4/(&npv*(&npv-1));
                               %end;
                                   %end;


            keep &byvar label impu_var
                        %if &proc = freq %then %do; percent frequency impu_var2 %end;

                        %else %do; estimate %end;
                        %if &proc = freq &&  &nivar > 1 %then %do; impu_var3 impu_var4 rowpercent colpercent
                                               %end;;
       run;
* Computes the overall variance ;
       data tot_var;merge smp_var impu_var;by &byvar label;

            tot_var = smp_var + impu_var;
            if tot_var =0 then fm=0;else
            fm = impu_var / tot_var;
            df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
            ddf = &df;
            df = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df = &ddf; %end; 
            tot_se  = sqrt(tot_var);
            %if &proc = freq %then %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(percent/tot_se),df));
                                       else prob_T=.;
                                       tot_var2 = smp_var2 + impu_var2;
                                       if tot_var2 =0 then fm=0;else
                                       fm = impu_var2 / tot_var2;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df2 = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df2 = &ddf; %end; 
                                       tot_se2  = sqrt(tot_var2);
                                       if tot_se2 ne 0 then prob_T2  = 2*(1-probt(abs(frequency/tot_se2),df2));
                                       else prob_T2=.;
                                       %if &nivar > 1 %then %do;
                                       tot_var3 = smp_var3 + impu_var3;
                                       if tot_var3 =0 then fm=0;else
                                       fm = impu_var3 / tot_var3;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df3 = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df3 = &ddf; %end; 
                                       tot_se3  = sqrt(tot_var3);
                                       if tot_se3 ne 0 then prob_T3  = 2*(1-probt(abs(rowpercent/tot_se3),df3));
                                       else prob_T3=.;
                                       tot_var4 = smp_var4 + impu_var4;
                                       if tot_var4 =0 then fm=0;else
                                       fm = impu_var4 / tot_var4;
                                       df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
                                       ddf = &df;
                                       df4 = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df4 = &ddf; %end; 
                                       tot_se4  = sqrt(tot_var4);
                                       if tot_se4 ne 0 then prob_T4  = 2*(1-probt(abs(colpercent/tot_se4),df4));
                                       else prob_T4=.;
                                                           %end;
                                   %end;

            %else %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),df));
                                       else prob_T=.;
                  %end;
       run;
                     %end;
    %else %do;
       data tot_var;set smp_var;
            tot_var = smp_var;
            df = &df;
            tot_se  = sqrt(tot_var);
            %if &proc = freq %then %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(percent/tot_se),&df));
                                       else prob_T=.;
                                       tot_var2 = smp_var2;
                                       df2 = &df;
                                       tot_se2  = sqrt(tot_var2);
                                       if tot_se2 ne 0 then prob_T2  = 2*(1-probt(abs(frequency/tot_se2),&df));
                                       else prob_T2=.;
                                       %if &nivar > 1 %then %do;
                                       tot_var3 = smp_var3;
                                       df3 = &df;
                                       tot_se3  = sqrt(tot_var3);
                                       if tot_se3 ne 0 then prob_T3  = 2*(1-probt(abs(rowpercent/tot_se3),&df));
                                       else prob_T3=.;
                                       tot_var4 = smp_var4;
                                       df4 = &df;
                                       tot_se4  = sqrt(tot_var4);
                                       if tot_se4 ne 0 then prob_T4  = 2*(1-probt(abs(colpercent/tot_se4),&df));
                                       else prob_T4=.;
                                                            %end;
                                   %end;

            %else %do;
                                       if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),&df));
                                       else prob_T=.;
                  %end;
       run;
          %end;

    %if &international = 1 && &nbyvar = 1 && &byvar = &cntryno && &proc ~= freq && &proc ~= corr && 
                                                                             &proc ~= univariate && %lowcase(&method) ~= diff %then %do;
        
        proc summary data=tot_var nway; class label; 
                     var estimate smp_var %if &npv=1 %then %do;%end;%else %do;impu_var %end;;
                     output out = international mean = estimate 
                                                n=n 
                                                sum= dummy smp_var  %if &npv=1 %then %do;%end;%else %do;impu_var %end;;  
        
        data international;set international;
             %if &npv > 1 %then %do;
             tot_var = (smp_var + impu_var) / (n*n);
             if tot_var =0 then fm=0;else  
             fm = (impu_var/(n*n)) / tot_var;
             df = 1/(fm*fm/(&npv-1) + (1-fm)*(1-fm)/&df);
             ddf = &df;
             df = min(df,ddf); %if %length(%scan(&ddf,1)) > 0 %then %do; df = &ddf; %end;  
             tot_se  = sqrt(tot_var);
             if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),df));
             else prob_T=.;
                                %end;
             %else %do;
             tot_var = smp_var/(n*n);
             df = &df;
             tot_se  = sqrt(tot_var);
             if tot_se ne 0 then prob_T  = 2*(1-probt(abs(estimate/tot_se),&df));
             else prob_T=.;
                   %end;
             drop n dummy;

        data tot_var;set tot_var international;run;
                                                                                           %end;
        


    proc sort data=tot_var;by &byvar label; run;
    data finalb(keep = &byvar label
                              %if &proc = catmod | &proc = logistic %then %do; odds_ratio lower upper %end;
                              %if &proc = freq %then %do; percent tot_se  df prob_T frequency tot_se2  df2 prob_t2
                                                          %if &nivar > 1 %then %do;
                                                          rowpercent tot_se3 colpercent tot_se4  df3 df4 prob_t3 prob_t4
                                                                               %end;
                                                                             );
                                                     %end;

                              %else %do; estimate tot_se  df prob_T); %end;
         merge  tot_var ;
         by &byvar;

         %if &proc = catmod | &proc = logistic %then %do;
             if  tot_se=0 then do;odds_ratio=.;LOWER=.;UPPER=.;end;
             else do;odds_ratio = exp(estimate);LOWER=exp(estimate-tot_se*tinv(&clev,df));
                                                UPPER=exp(estimate+tot_se*tinv(&clev,df));
                  end;
                                                   %end;
         output finalb;
    run;

    proc print data=finalb label;
    %if &proc=freq %then %do;
        var &byvar label frequency tot_se2 prob_t2 df2 percent tot_se  prob_t df 
                             %if &nivar>1 %then %do;
                                                    rowpercent tot_se3 prob_t3 df3 colpercent tot_se4 prob_t4 df4
                                                %end;;
        format frequency 10. tot_se2 10. df2 5.2 prob_t2 7.5 percent 5.1 tot_se 6.2 df 5.2 prob_t 7.5
                             %if &nivar>1 %then %do;
                                                    rowpercent 5.1 tot_se3 6.2  df3 5.2 prob_t3 7.5 colpercent 5.1 tot_se4 6.2 df4 5.2 prob_t4 7.5
                                                %end;;

                         %end;
    %else %do;
        var &byvar label estimate tot_se prob_t df;
        format estimate 10.2 tot_se 10.3 df 5.2 prob_t 7.5;
        %if &international = 1 %then %do; format %scan(&byvar,1) int.;%end;  
          %end;  
    label
         Tot_se = "Standard Error"
         prob_T = "Prob > |T|"
         df = "D.F."
         %if &proc=freq %then %do;
             Tot_se2 = "Standard Error"
             prob_T2 = "Prob > |T|"
             df2 = "D.F."
             %if &nivar>1 %then %do;
                 Tot_se3 = "Standard Error"
                 prob_T3 = "Prob > |T|"
                 df3 = "D.F."
                 df4 = "D.F." 
                 Tot_se4 = "Standard Error"
                 prob_T4 = "Prob > |T|"
                               %end;
                              %end;

         %if &proc = catmod | &proc = logistic %then %do;
         odds_ratio ="Odds Ratio"
         LOWER = "95% Wald Lower Limit"
         UPPER = "95% Wald Upper Limit"
     
                                                   %end;;
     title4 "Comparison(s) between cells";
    run;

%end;
                                                         %end;

%mend;

ods listing;
run;

/*
%stattool(wgt=popwt, nrep=30, npv=1, method=diff, infile=in, ref=1 1,
                                                             ivar = gendaa2 age3   , ivarpv = 0 0,
                                                             dvar = prose1    , dvarpv =0,
                                                             clvar= gendaa2 age3  , clvarpv=0 0);


%stattool(wgt=popwt, nrep=30, npv=1, method=crosstabs, infile=in, ivar=gendaa2 age3, ivarpv=0 0);



*/
